% DETECTSHOCK.M - cubic fit to detect shock formation
% Program to test shock detection using a cubic fit
% Refer to Section 2.9 "Detection of Shock Formation"
%  A SHOCK FITTING PRIMER - CHAPTER 2, "A More Difficult Example"
%
%  Copyright 2005-2009 M.D.Salas.

clear all

disp('A SHOCK-FITTING PRIMER')
disp(' ')

N=400;
K=1000;
dx=6/N;
time0=0;
tadd=.01;
ts=sqrt(exp(1)/2);
dt=(ts+tadd)/K;
useed=0;
sig=1;
nfig=0;
du(1)=0;

for n=1:N+1
    x(n)=-3+dx*(n-1);
end

for k=1:K+1
    t=time0+dt*(k-1);

    for n=1:N+1
        u(n)=gauss_init(x(n),t,useed,sig,n);                                % function that models shock profile as a function of (x,t)
        useed=u(n);                                                         % this represents the numerical calculation of the u-profile
    end

    for n=2:N+1                                                             
        du(n)=u(n)-u(n-1);
    end

    [dumin,ns]=min(du);                                                     % find max |du/dx| & du <0

    x1=x(ns-2);
    u1=u(ns-2);
    u2=u(ns-1);
    u3=u(ns);
    u4=u(ns+1);
    
    [a,b,c,d]=cubic_coeff(x1,dx,u1,u2,u3,u4);                               % compute cubic coefficients

    det(k)=b^2-3*a*c;
    tplot(k)=t;
    useed=u(1);
    
end

for k=1:K
    if det(k) > 0
        disp(['shock detected at t= ' num2str(tplot(k))...
            ', dt= ' num2str(dt)])
        disp(['the exact solution predicts t= ' num2str(ts)])
        disp([])
        ushock=-b/(3*a);
        us=1/sqrt(exp(1));
        xs=sqrt(2);
        xshock=a*ushock^3+b*ushock^2+c*ushock+d;
        disp(['the exact solution predicts us= ' num2str(us)...
            ', the computed us= ' num2str(ushock)])
        disp(['the exact solution predicts xs= ' num2str(xs)...
            ', the computed xs= ' num2str(xshock)])
        break
    end
end

nfig=nfig+1;
figure(nfig)                                                                % check that curve fit goes through data points
plot(u,x,'k-')
hold on
y1=a*u1^3+b*u1^2+c*u1+d;
y2=a*u2^3+b*u2^2+c*u2+d;
y3=a*u3^3+b*u3^2+c*u3+d;
y4=a*u4^3+b*u4^2+c*u4+d;
plot(u1,y1,'ko',u2,y2,'ko',u3,y3,'ko',u4,y4,'ko')
title('curve fit and data points used for fit')
xlabel('u')
ylabel('x')
hold off

nfig=nfig+1;
figure(nfig)                                                                % reverse previous plot
plot(x,u,'k-')
hold on
plot(y1,u1,'ko',y2,u2,'ko',y3,u3,'ko',y4,u4,'ko')
xlabel('x')
ylabel('u')
hold off

tp1(1)=0;
tp1(2)=1.4;
yp(1)=0;
yp(2)=0;
nfig=nfig+1;
figure(nfig)                                                                % plot determinant, b^2-3ac
plot(tplot,det,'k-', ts,0,'ko',tp1,yp,'k-')
xmin=0;
xmax=1.4;
xlim([xmin,xmax])
xlabel('t')
ylabel('b^2-3ac')

for n=1:N+1
    v(n)=-1+dx*(n-1);
    y(n)=a*v(n)^3+b*v(n)^2+c*v(n)+d;
    yv(n)=3*a*v(n)^2+2*b*v(n)+c;
end

nfig=nfig+1;
figure(nfig)                                                                % compares fit to original curve
plot(v,y,'k--',u,x,'k-')
grid on
xmin=-3;
xmax=3.;
xlim([xmin,xmax])
ymin=0;
ymax=1.;
xlim([ymin,ymax])
xlabel('u')
ylabel('x')
legend('curve fit', 'original curve')

nfig=nfig+1;
figure(nfig)                                                                % plots dx/du vs u
plot(v,yv,'k-')
grid on
xlabel('u')
ylabel('dx/du')
ymin=-1;
ymax=1;
ylim([ymin ymax])




