%  EXACTGAUSS.M - Finds shock and jumps for a Gaussian pulse. 
%
%  Finds the shock location "xs" at a given t for the inviscid Burger's 
%  equation with a Gaussian initial conditions ( u(x,t)=e^-(x^2-u*t) ).
%  There is no solution for t < sqrt(e/2). For t > 10, the data for
%  guessing ul (u_left) and ur (u_right) would need to be updated.
%
%  Dependency: solve_for_ur.m
%  A SHOCK FITTING PRIMER - CHAPTER 2, "A More Difficult Example"
%
%  Copyright 2005-2009 M.D.Salas.


clear all

global ur0 eps0 ul ur sqpi t gr gl

e=exp(1);
ts=sqrt(e/2); % time for shock formation
disp('A SHOCK-FITTING PRIMER')
disp(' ')
disp('In the next line enter the time for exact solution')
t=input('Enter time, (1.1659 <= t) & (t <= 10.)> ');                        % user inputs t
if (t >= ts) && (t <= 10.)                                                  % if not in range stop
    sqpi=sqrt(pi);
    us=exp(-1/2);
    c1=e*sqrt(2*e);
    c2=-e^2*sqrt(2);
    eps0=10.^-9;                                                            % set tolerance

    tdata= [1.1658 1.1750 1.1800 1.1903 1.2000 1.3000 1.3500 1.4000...
        1.4500 1.5000 1.5500 1.6000 1.7421 1.7670 1.8000 2.0000 2.2500 ...
        2.5000 3.0000 3.5000 4.0000 4.5000 5.0000 6.0000 ...
        7.0000 8.0000 10.0000];
    uldata=[0.6065 0.69553 0.71653 0.74653 0.7752 0.9008 0.9322 0.9543...
        0.9701 0.9814 0.9893 0.9946 1.0000 0.9999 0.9993 0.9897 0.9692 ...
        0.9452 0.8962 0.8509 0.8106 0.7750 0.7434 0.6899 ...
        0.6463 0.6101 0.5528];
    urdata=[0.6065 0.51004 0.48601 0.44674 0.4274 0.2732 0.2286 0.1938...
        0.1659 0.1430 0.1239 0.1079 0.0742 0.0696 0.0641 0.0393 0.0219 ...
        0.0124 0.0041 0.0013 4.3396e-004 1.3758e-004 4.2696e-005...
        3.8753e-006 3.2862e-007 2.6343e-008 1.4791e-010];
    xsdata=[1.4142 1.4198 1.4229 1.4293 1.4349 1.4943 1.5234 1.5523...
        1.5809 1.6091 1.6371 1.6648 1.7421 1.7554 1.7730 1.8776 2.0038 ...
        2.1256 2.3576 2.5765 2.7843 2.9825 3.1722 3.5300 ...
        3.8637 4.1776 4.7576];

    ul0=interp1(tdata,uldata,t,'linear');                                   % guess ul
    ur0=interp1(tdata,urdata,t,'linear');                                   % guess ur

    if ur0 < 0.
        ur0=eps0;
    end
                                                                            % Guess ul. For a guessed ul, find ur by solving the 2nd of (2.72) or
                                                                            % (2.73). With ur known, see if the guessed ul satisfies 1st of (2.72)
                                                                            % or (2.73), if not iterate on ul.   
    tuo=zeros(2);
    eo=zeros(2);
    for n=1:20                                                              % outer  loop for ul
        if n==1
            tuo(1)=ul0;
            mo=1;
        end
        if n==2
            tuo(2)=ul0*.98;
            mo=2;
        end
        if n>2
            tuo3=tuo(2)-eo(2)*(tuo(2)-tuo(1))/(eo(2)-eo(1));
            tuo(1)=tuo(2);
            eo(1)=eo(2);
            tuo(2)=tuo3;
        end
        ul=tuo(mo);
        if ul > 1.
            ul=1.;
            tuo(mo)=1.;
        end
                                                                            % inner loop
                                                                            % solve for ur for a given ul using 2nd equation of (2.72) or (2.73)
        em=solve_for_ur;

        if t < 1.2                                                          % outer loop; solve 1st of (2.72) or (2.73) for ul given t
            dur=ur-us;
            dul=ul-us;
            dur2=dur^2;
            dul2=dul^2;
            dt=c1*(dur2+dul2+dur*dul)/6.+c2*(dur2+dul2)*(dur+dul)/24.;
            eo(mo)=t-ts-dt;
        elseif t > 1.7421
            eo(mo)=t-(gr+gl)/(ul-ur);
        elseif (t >= 1.2) && (t <= 1.7421)
            eo(mo)=t-(gr-gl)/(ul-ur);
        end
        if abs(eo(mo)) < eps0;
            break
        end
    end                                                                     % end of outer "for n" loop
    if abs(eo(mo)) > eps0
        disp(['the error in "ul" iteration is: ', num2str(eo(mo)),...
            'at t= ', num2str(t)])
    end

    if t > 1.7421
        xs=ul*t-gl;
    else
        xs=ul*t+gl;
    end
                                                                            %----------------------OUTPUT---------------------
    disp(' ')
    disp('the error in "t" iteration is:'), disp(eo(mo))
    disp('the error in "ur" iteration is:'), disp(em)
    format long
    t, xs, ul, ur
    format short

    figure(1)                                                               % plot shock jump envelope
    plot(tdata, uldata, 'k-', tdata, urdata, 'r-', t, ur, '+', t, ul, '+')
    axis([1 10 -0.1 1.1])
    xlabel('time', 'FontSize', 12)
    ylabel('UL & UR', 'FontSize', 12)
    title('Plot of Shock Jump Envelope', 'FontSize', 12)

    figure(2)                                                               % plot velocity field at time t
    
    for n=1:100
        ucheck=n/100;
        if t > xs
            if ucheck < ul
                um(n)=ucheck;
                nlast=n;
            else
                break
            end
        else
            um(n)=ucheck;
            nlast=n;
        end
    end
    for n=1:nlast;
        xm(n)=um(n)*t-sqrt(-log(um(n)));
    end
    if t > xs
    else
        nlasto=nlast;
        nlast=nlast+9;
        du=(um(nlasto)-ul)/10.;
        for n=nlasto+1:nlast
            um(n)=um(nlasto)-du*(n-nlasto);
            xm(n)=um(n)*t+sqrt(-log(um(n)));
        end
    end
    xm(nlast+1)=xs;
    um(nlast+1)=ul;
    xm(nlast+2)=xs;
    um(nlast+2)=ur;
    up(1)=ur;
    xp(1)=xs;
    for n=2:100
        ucheck=ur*(1.-n/100);
        if ucheck < ur
            up(n)=ucheck+eps0;
            nlast=n;
        else
            break
        end
    end
    for n=2:nlast;
        xp(n)=up(n)*t+sqrt(-log(up(n)));
    end
    plot(xm, um, 'k-', xp, up,'k-')
    xlabel('X','FontSize', 12)
    ylabel('U','FontSize', 12)

    figure(3)                                                               % plot shock location on shock-path
    plot(tdata, xsdata, 'k-', t, xs, '+')
    xlabel('time', 'FontSize', 12)
    ylabel('shock path', 'FontSize', 12)
    title('Plot of Shock Path', 'FontSize', 12)
else
    disp('t must be >= to 1.1659 and <= 10.')                               % not in allowed range
end
