% SHOCKCAP01.M - shock caturing scheme
% A SHOCK FITTING PRIMER - CHAPTER 2, "Application of Background Scheme".
% CHAPTER 2, Numerical integration of inviscid Burgers equation with
% Gaussian or "one-saw-tooth" initial conditions.
% Uses a 3 stage, low-storage, Runge-Kutta scheme, with Roe's approximate
% Riemann solver and Kappa interpolation with limiter.
% This program makes function calls to:
%
%                      function u=gauss_bc(x,t,u)
%                      function plot_solution_cap01
%                      function exactgauss_xs
%                      function gauss_init
%                      function solve_for_ur
%
% These file must reside in the same directory as this file.
% Code checking: with INIT=2, tlast=5, CN=1, LIMITER=1, N=400, xr=2.8, 
% xl=-1.5 kplot=1 => After 362 time steps, at time= 5, xs= 2.4506
%
% Copyright 2005-2009 M.D.Salas.

clear all 

global t tshock nfig kn ts xs INIT x dx u N NP kout

disp('A SHOCK-FITTING PRIMER')
disp(' ')
disp('INIT=1 Gaussian, INIT=2 "one-saw-tooth"')
disp('In the next line enter the case to be executed with default values')
INIT=input('Enter INIT (1 or 2) ');

if INIT ~= 1 || INIT ~= 2
    disp('INIT not in range; it was reset to 1')
    INIT=1;
end
                                                        		            % INIT=1 Gaussian, INIT=2 "one-saw-tooth"
tlast=5;                                                                    % runs to a max time of t=tlast
CN=1;                                                                       % CN=Courant Number
LIMITER=1;                                                                  % LIMITER=1 -> on, LIMITER=0 -> off
Ns=[100 400];                                                               % number of mesh points minus one; NS=Ns(INIT)
xrs=[4. 2.8];                                                               % xr=right boundary, xr=3 Gaussian, xr=2.8 "one-saw-tooth"	
xls=[-3. -1.5];                                                             % xl=left boundary, xl=-3 Gaussian, xl=-1.5 "one-saw-tooth"
kappa=1/3;                                                                  % kappa=Kappa interpolation parameter
kplot=1;                                                                    % plot shock location every kplot steps
                                                                            %               NO CHANGES BELOW THIS LINE
                                                                            %               HOUSEKEEPING & INITIAL CONDITIONS
t=0.;                                                                       % initial time
tshock=[1.1658 1.0];                                                        % tshock=exact time for shock formation
xr=xrs(INIT);
xl=xls(INIT);
Length=xr-xl;                                                               % computational domain
N=Ns(INIT);                                                                 % N=number of mesh points -1
NM=N-1;
NP=N+1;
dx=Length/N;                                                                % mesh interval size
eps0=10^-7;
km=1-kappa;
kp=1+kappa;
a=[1./3. .5 1.];                                                            % a(k)=Runge-Kutta coefficients
sqpi=sqrt(pi);
x=zeros(1,NP);                                                              % preallocate memory
u=zeros(1,NP);	
uo=zeros(1,NP);	
f=zeros(1,NP);	
q=zeros(1,NP);
drs=zeros(1,NP);
nfig=0;

for n=1:NP
    x(n)=xl+(n-1)*dx;
end
if INIT == 1                                                                  % initial conditions for Gaussian
    c=sqpi/(2.*dx);
    for n=1:NP
        u(n)=c*(erf(x(n)+.5*dx)-erf(x(n)-.5*dx));                           % cell averaged values for u(x)=exp(-x^2)
    end
else                                                                        % initial conditions for "one-saw-tooth"
    for n=1:NP
        if (x(n) > -1) && (x(n) <= 0.)
            u(n)=1.+x(n);
        elseif (x(n) > 0.) && (x(n) < 1.)
            u(n)=1.-x(n);
        end
    end
end 

LAST=0;
kout=0;
kn=0;
SAFETY=1000;
                                                                            %             MAIN INTEGRATION LOOP
while LAST == 0                                                             % begin time stepping loop
    kout=kout+1;
    
    if kout > SAFETY
        LAST=1;
        disp('Stopped because possible out-of-control condition')
        disp('Check SAFETY value')
    end
    
    [umax]=max(abs(u));
    dt=CN*dx/umax;                                                          % CFL condition
    
    if (t+dt) >= tlast                                                      % check that time does not exceed tlast
        dt=tlast-t;
        LAST=1;                                                             % trigger to stop while loop
    end
    
    sigma=dt/dx;
    uo=u;                                                                   % save latest values of u for use in RK scheme 
    
    for k=1:3                                                               % begin 3 stage RK loop 
        asig=a(k)*sigma;
        du=diff(u);                                                         % du(n)=u(n+1)-u(n)
        
        if LIMITER == 1                                                     % limiter on         
            for n=2:NM
                s=(2*du(n)*du(n-1)+eps0)/(du(n)^2+du(n-1)^2+eps0); 
                ks=kappa*s;
                ul=u(n)+.25*s*((1-ks)*du(n-1)+(1+ks)*du(n));                % Kappa interpolation
                ur=u(n+1)-.25*s*((1-ks)*du(n)+(1+ks)*du(n+1)); 
                f(n)=.25*(ul^2+ur^2-abs(ul+ur)*(ur-ul));                    % evaluate Roe flux
            end
        else
            for n=2:NM                                                      % limiter off
                ul=u(n)+.25*(km*du(n-1)+kp*du(n));                          % Kappa interpolation           
                ur=u(n+1)-.25*(kp*du(n)+km*du(n+1));                        % Kappa interpolation          
                f(n)=.25*(ul^2+ur^2-abs(ul+ur)*(ur-ul));                    % evaluate Roe flux
            end
        end
        
        df=diff(f);                                                         % df(n)=f(n+1)-f(n)
        for n=2:NM                                                          % do RK integration
            u(n)=uo(n)-asig*df(n-1);
        end 
        
        if INIT == 1                                                        % update boundary values for Gaussian initial conditions
            u(1)=gauss_init(x(1),t+a(k)*dt,u(1),1,1);                       % inflow, exact solution
            u(2)=gauss_init(x(2),t+a(k)*dt,u(2),1,2);
            u(N)=2*u(N-1)-u(N-2);                                           % outflow, extrapolate
            u(NP)=2*u(NP-1)-u(NP-2);         
        end
        
    end                                                                     % end 3 stage RK loop
  
    t=t+dt;                                                                 % increment time 
    
    test=rem(kout,kplot);                                                   % plot every kplot step
    if test == 0
        if t > tshock(INIT)                                                 % locate shock   
            kn=kn+1;                                                        % count no. of shock points
            du=diff(u);                                                     % delta u
            du(1)=0.;
            du(NP)=0.;            
            [dumax,nmax]=max(abs(du));                                      % use max(|delta u|) to locate shock
            xs(kn)=.5*(x(nmax)+x(nmax-1));
            ts(kn)=t;
        end  
    end
    
end		                            	% end time stepping "while" loop

%                           OUTPUT

plot_solution_cap01


