% SHOCKFIT01.M - Shock-fitted, one region.
% A SHOCK FITTING PRIMER - CHAPTER 2, "Boundary shock-fitting".
% CHAPTER 2, Numerical integration of inviscid Burgers equation with
% "one-saw-tooth" initial conditions at t=1. Uses a 3 stage, low-storage,
% Runge-Kutta scheme, with Roe fluxes and Kappa interpolation.
%
% Coding check: with tlast=5, CN=1, xl=-1.5, N=100 => xs=2.4680
% Copyright 2005-2009 M.D.Salas.

clear all

disp('A SHOCK-FITTING PRIMER')
disp(' ')

tlast=5;                                                                    % runs to a max time of t=tlast
CN=1.;                                                                      % CN=Courant Number
xl=-1.5;                                                                    % left boundary
N=100;                                                                      % N=number of mesh points -1
kappa=1./3.;                                                                % kappa=Kappa interpolation parameter
xs=1.;                                                                      % initial shock position, right boundary
t=1.;                                                                       % initial time
usL=1.;                                                                     % u at shock (left side)
LIMITER=0;                                                                  % LIMITER=0 -> off, LIMITER=1 -> on
eps0=10.^(-9);
                                                                            %------------NO CHANGES BELOW THIS LINE-------------------------
NM=N-1;
NP=N+1;
w=.5*usL;                                                                   % shock speed at t=1.
zx=1./(xs-xl);                                                              % Jacobian
dz=1./N;                                                                    % mesh interval size
kp=1.+kappa;
km=1.-kappa;
a=[1./3. .5 1.];                                                            % a(k)=Runge-Kutta coefficients
x=zeros(1,N+2);                                                             % preallocated memory for x
z=zeros(1,N+2);                                                             % preallocated memory for z
u=zeros(1,N+2);                                                             % preallocated memory for u
Uo=zeros(1,N+2);                                                            % preallocated memory for uo
U=zeros(1,N+2);                                                             % preallocated memory for U
F=zeros(1,N+2);                                                             % preallocated memory for F
for n=1:N+2                                                                 % define initial conditions, t=1.
    z(n)=(n-1)*dz;                                                          % computational coordinate [0,1]
    x(n)=xl+z(n)/zx;                                                        % physical coordinate [xl,xs]
    if (x(n) > -1)
        u(n)=(1.+x(n))/(1.+t);
    else
        u(n)=0.;
    end
end

U=u/zx;
LAST=0;
kout=0;
SAFETY=1000;

while LAST == 0                                                             % begin time stepping loop
    kout=kout+1;
    
    if kout > SAFETY
        LAST=1;
        disp('Stopped because possible out-of-control condition')
        disp('Check SAFETY value')
    end
    
    [umax]=max(abs(u));
    dt=CN*dz/zx/umax;                                                       % CFL condition
    sigma=dt/dz;
    Uo=U;                                                                   % save latest values for use in RK scheme
    xso=xs;
    wo=w;
    F(1)=0.;                                                                % set flux at left boundary to zero
    for k=1:3                                                               % begin 3 stage RK loop
        asig=a(k)*sigma;
        du=diff(u);
        if LIMITER == 1                                                     % LIMITER on
            for n=2:N
                s=(2*du(n)*du(n-1)+eps0)/(du(n)^2+du(n-1)^2+eps0);
                ks=kappa*s;
                ul=u(n)+.25*s*((1-ks)*du(n-1)+(1+ks)*du(n));                % Kappa interpolation
                ur=u(n+1)-.25*s*((1-ks)*du(n)+(1+ks)*du(n+1));
                zav=.5*(z(n)+z(n+1));
                uav=.5*(ur+ul);
                F(n)=.25*(ul^2+ur^2-abs(ul+ur)*(ur-ul))-zav*uav*w;          % evaluate Roe flux
            end
        else
            for n=2:N                                                       % LIMITER off
                ul=u(n)+.25*(km*du(n-1)+kp*du(n));                          % Kappa interpolation
                ur=u(n+1)-.25*(kp*du(n)+km*du(n+1));                        % Kappa interpolation
                zav=.5*(z(n)+z(n+1));
                uav=.5*(ur+ul);
                F(n)=.25*(ul^2+ur^2-abs(ul+ur)*(ur-ul))-zav*uav*w;          % evaluate Roe flux
            end
        end
        df=diff(F);
        for n=2:N                                                           % compute residual and perform RK integration
            U(n)=Uo(n)-asig*(df(n-1));
        end
                                                                            % integrate characteristic reaching shock
        uz=(u(NP)-u(N))/dz;
        R=-.5*w*zx*uz;                                                      % residual
        xs=xso+a(k)*w*dt;                                                   % RK integration for shock location
        w=wo+a(k)*R*dt;                                                     % RK integration for shock speed
        usL=2.*w;                                                           % jump condition for u at shock
        zx=1./(xs-xl);                                                      % update Jacobian
        u(NP)=usL;
        U(NP)=u(NP)/zx;
        n=1:N;                                                              % update u
        u(n)=U(n)*zx;
        u(NP+1)=2*u(NP)-u(N);                                               % extrapolate to ghost point to right of shock
    end                                                                     % end 3 stage RK "for" loop
    t=t+dt;                                                                 % increment time
    if t > tlast                                                            % stop if t > tlast
        LAST=1;                                                             % trigger to stop computation
        told=t-dt;
        t=tlast;
        dt=t-told;
    end
end                                                                         % end time stepping "while" loop
                                                                            %----------------------OUTPUT--------------------------
disp('Output from SHOCKFIT01:')
disp(' ')
disp(['In region 1 N= ' num2str(N) ])
disp(' ')
disp(['After ' num2str(kout) ' time steps:'])
disp(['time= ' num2str(t) ' and xs= ' num2str(xs)])
n=1:NP;
x(n)=xl+z(n)/zx;
u(NP+1)=0.;
x(NP+1)=xs+.2;
xsexact=sqrt(2*(1+t))-1.;                                                   % exact shock position at t=tlast
error=1. -xs/xsexact ;                                                      % error in shock position
usexact=sqrt(2*(1+t))/(1+t);                                                % exact u at the shock
disp(['exact shock position= ' num2str(xsexact)...
    ' error= ' num2str(error) ])
xe=[-1.2    -1.     xsexact     xsexact     xsexact+.2];
ue=[0.      0.      usexact     0.          0.];
figure(1)
plot(x,u,'ko',xe,ue,'k-',xs, 0.,'k-')
axis([xl,xs+.2,-.1,usexact+.1])
xlabel('x')
ylabel('u')
legend('computed','exact')
hold on
error=0;
                                                                            % re-normalize x to compute error
for n=1:N
    xplot2(n)=xl+z(n)*(xsexact-xl);
    uexact=0;
    if (xplot2(n) > -1) && (xplot2(n) <= xsexact)
        uexact=(1+xplot2(n))/(1+t);
    end
    drs(n)=abs(u(n)-uexact);
    error=error+drs(n);
end
[drmax,nmax]=max(abs(drs));
error=error/N;
disp(' ')
disp(['In region 1 the average error in u is ' num2str(error) ])
disp(['The max error in u is ' num2str(drmax) ' at n= ' num2str(nmax)])
for n=1:N
    drs(n)=.5*drs(n)/(drmax+10^-12);
end
plot(xplot2,drs,'b--')
hold off