function mac_non_conservative
% MAC_NON_CONSERVATIVE.M solver based on the MacCormack's Scheme
% A SHOCK FITTING PRIMER - CHAPTER 2, "Boundary Shock-Fitting".
%
% modified for upwinding
%
% Copyright 2005-2008 M.D.Salas.

global kout eps0 z dz zx zt
global xLEFT kplot tout xsout sig
global Nlast N NT INIT CN
global u U xs w usL usR t tlast LAST xl xr

SAFETY=1000;                                                                % stops calculation if over run condition

u(1,NT(1))=0;                                                               % shock ghost points not needed
u(2,1)=0;

while LAST == 0                                                             % begin time stepping loop

    kout=kout+1;                                                            % step counter

    if kout > SAFETY
        LAST=1;
        disp('Stopped because possible out-of-control condition')
        disp('Check SAFETY value')
    end

    for m=1:2                                                               % begin CFL condition
        Amax=eps0;
        for n=2:Nlast(m)
            A=zt(m,n)+zx(m)*u(m,n);
            Aabs=abs(A);
            if Aabs > Amax
                Amax=Aabs;
            end
        end
        dtm(m)=dz(m)/Amax;
    end
    dt=CN*min([dtm(1), dtm(2)]);                                            % end CFL condition

    if (t+dt) >= tlast                                                      % check that time does not exceed tlast
        dt=tlast-t;
        LAST=1;                                                             % trigger to stop while loop
    end

    Uo=u;                                                                   % save latest values
    xso=xs;
    wo=w;
    usRo=usR;
    usLo=usL;

    for k=1:2                                                               % predictor-corrector loop
        fac=k/2;
        np=2-k;
        nm=1-k;
        for m=1:2                                                           % for regions 1 & 2
            sigma=dt/dz(m);

            for n=(m+1):N(m)+(m-1)
                A=zt(m,n)+zx(m)*u(m,n);
                uz=u(m,n+np)-u(m,n+nm);
                ut=-A*uz;
                if k == 1
                    U(m,n)=u(m,n)+sigma*ut;
                else
                    U(m,n)=.5*(Uo(m,n)+u(m,n)+sigma*ut);
                end
            end

        end                                                                 % end "for m" region 1 & 2 loop

        if k == 1                                                           % shock point computation
            uzL=(u(1,Nlast(1))-u(1,N(1)))/dz(1);
            uzR=(u(2,3)-u(2,2))/dz(2);
            RuL=-(usL-w)*zx(1)*uzL;
            RuR=-(usR-w)*zx(2)*uzR;
            R=.5*(RuL+RuR);
            xs=xso+w*dt;                                                    %  predictor for shock location
            w=wo+R*dt;                                                      %  predictor for shock speed
            if INIT == 1 || INIT == 2
                usL=usLo+RuL*dt;
                usR=2.*w-usL;
            else                                                            % INIT==4 | INIT==3
                usR=usRo+RuR*dt;
                usL=2.*w-usR;
            end
        else
            uzL=(3*u(1,Nlast(1))-4*u(1,N(1))+u(1,N(1)-1))/(2*dz(2));
            uzR=-(3*u(2,2)-4*u(2,3)+u(2,4))/(2*dz(2));
            RuL=-(usL-w)*zx(1)*uzL;
            RuR=-(usR-w)*zx(2)*uzR;
            R=.5*(RuL+RuR);
            xs=.5*(xso+xs+w*dt);                                            %  corrector for shock location
            w=.5*(wo+w+R*dt);                                               %  corrector for shock speed
            if INIT == 1 || INIT == 2
                usL=.5*(usLo+usL+RuL*dt);
                usR=2.*w-usL;
            else                                                            % INIT==4 | INIT==3
                usR=.5*(usRo+usR+RuR*dt);
                usL=2.*w-usR;
            end
        end
        zx(1)=1./(xs-xl);                                                   % update Jacobian, region 1
        zx(2)=1./(xr-xs);                                                   % update Jacobian, region 2
        xLEFT(2)=xs;                                                        % left boundary region 2
        U(1,Nlast(1))=usL;
        U(2,2)=usR;

        if INIT == 1                                                        % update boundary values for Gaussian initial conditions
            U(1,1)=gauss_init(xl,t+fac*dt,u(1,1),sig,1);                    % inflow, exact solution
            U(2,NT(2))=2*U(2,Nlast(2))-U(2,N(2));                           % extrapolate outflow
        elseif INIT == 4                                                    % update boundary values for Gaussian initial conditions
            U(2,NT(2))=gauss_init(xr,t+fac*dt,u(2,NT(2)),sig,NT(2));        % inflow, exact solution
            U(1,1)=2*U(1,2)-U(1,3);                                         % extrapolate outflow
        elseif INIT == 2
            U(2,NT(2))=0;
        elseif INIT == 3
            U(2,NT(2))=0;
        end

        for n=1:Nlast(1);                                                   % update, region 1
            zt(1,n)=-z(1,n)*zx(1)*w;
            u(1,n)=U(1,n);
        end

        for n=2:NT(2);                                                      % update, region 2
            zt(2,n)=(z(2,n)-1)*zx(2)*w;
            u(2,n)=U(2,n);
        end

    end                                                                     % end predictor-corrector loop

    t=t+dt;                                                                 % increment time

    test=rem(kout,kplot);                                                   % save values for shock location plot every kplot steps
    if test==0
        tout(kout)=t;
        xsout(kout)=xs;
    end

end                                                                         % end time stepping "while" loop
