function roe_runge_kutta
% ROE_RUNGE_KUTTA.M solver based on Roe's approximate Riemann solver
% and 3 stage, low storage, Runge Kutta time integration
% A SHOCK FITTING PRIMER - CHAPTER 2, "Boundary Shock-Fitting".
%
% Copyright 2005-2008 M.D.Salas.

global kout LIMITER kappa eps0 z dz zx zt
global xLEFT kplot tout xsout sig
global Nfirst Nlast N NT INIT CN
global u U xs w usL usR t tlast LAST xl xr

kp=1.+kappa;
km=1.-kappa;
a=[1/3 1/2 1];                                                              % a(k)=Runge-Kutta coefficients

for m=1:2
    for n=1:NT(m);
        U(m,n)=u(m,n)/zx(m);
    end
end

SAFETY=1000;

while LAST == 0                                                             % begin time stepping loop
    
    kout=kout+1;                                                            % step counter
    
    if kout > SAFETY
        LAST=1;
        disp('Stopped because possible out-of-control condition')
        disp('Check SAFETY value')
    end    
    
    for m=1:2                                                               % begin CFL condition
        Amax=eps0;
        for n=2:Nlast(m)
            A=zt(m,n)+zx(m)*u(m,n);
            Aabs=abs(A);
            if Aabs > Amax
                Amax=Aabs;
            end
        end
        dtm(m)=dz(m)/Amax;
    end
    dt=CN*min([dtm(1), dtm(2)]);                                            % end CFL condition

    if (t+dt) >= tlast                                                      % check that time does not exceed tlast
        dt=tlast-t;
        LAST=1;                                                             % trigger to stop while loop
    end

    Uo=U;                                                                   % save latest values for use in RK scheme
    xso=xs;
    wo=w;
    usRo=usR;
    usLo=usL;

    for k=1:3                                                               % begin 3 stage RK loop

        for m=1:2                                                           % for regions 1 & 2
            asigma=a(k)*dt/dz(m);
            dum=u(m,2)-u(m,1);
            dup=u(m,3)-u(m,2);
            for n=2:N(m)
                dupp=u(m,n+2)-u(m,n+1);
                if LIMITER == 1
                    s=(2*dup*dum+eps0)/(dup^2+dum^2+eps0);
                    km=1-kappa*s;
                    kp=1+kappa*s;
                    ql=.25*s*(km*dum+kp*dup);
                    dum=dup;
                    dup=dupp;
                    s=(2*dup*dum+eps0)/(dup^2+dum^2+eps0);
                    km=1-kappa*s;
                    kp=1+kappa*s;
                    qr=.25*s*(kp*dum+km*dup);
                else
                    ql=.25*(km*dum+kp*dup);
                    dum=dup;
                    dup=dupp;
                    qr=.25*(kp*dum+km*dup);
                end
                ul=u(m,n)+ql;
                ur=u(m,n+1)-qr;
                ulr=ul+ur;
                ztav=.5*(zt(m,n)+zt(m,n+1))/zx(m);
                uav=.5*(u(m,n)+u(m,n+1));
                if (ul < 0) && (ur > 0)                                     % entropy fix
                    F(n)=ztav*uav;
                elseif ulr > 0                                              % evaluate Roe flux
                    if(u(m,n+1)-u(m,n)) < 0
                        uav=u(m,n+1);
                    end
                    F(n)=.5*ul^2+ztav*uav;
                else
                    if(u(m,n+1)-u(m,n)) < 0
                        uav=u(m,n);
                    end
                    F(n)=.5*ur^2+ztav*uav;
                end
            end

            for n=Nfirst(m):N(m)                                            % RK integration
                U(m,n)=Uo(m,n)-asigma*(F(n)-F(n-1));
            end
        end                                                                 % end "for m" region 1 & 2 loop

        uzL=(usL-u(1,N(1)))/dz(1);                                          % shock point computation
        RuL=-(usL-w)*zx(1)*uzL;
        uzR=(u(2,3)-usR)/dz(2);
        RuR=-(usR-w)*zx(2)*uzR;
        R=.5*(RuL+RuR);
        xs=xso+a(k)*w*dt;                                                   % RK integration for shock location
        w=wo+a(k)*R*dt;                                                     % RK integration for shock speed
        if INIT == 1 || INIT == 2
            usL=usLo+a(k)*RuL*dt;
            usR=2.*w-usL;
        elseif INIT == 4 || INIT == 3
            usR=usRo+a(k)*RuR*dt;
            usL=2.*w-usR;
        end
        zx(1)=1./(xs-xl);                                                   % update Jacobian, region 1
        zx(2)=1./(xr-xs);                                                   % update Jacobian, region 2
        xLEFT(2)=xs;                                                        % update left boundary region 2
        u(1,Nlast(1))=usL;
        u(2,2)=usR;
        U(1,Nlast(1))=u(1,Nlast(1))/zx(1);
        U(2,2)=u(2,2)/zx(2);
        u(1,NT(1))=2*u(1,Nlast(1))-u(1,N(1));                               % extrapolation for ghost point at shock
        u(2,1)=2*u(2,2)-u(2,3);

        if INIT == 1                                                        % update boundary values for Gaussian initial conditions
            u(1,1)=gauss_init(xl,t+a(k)*dt,u(1,1),sig,1);                   % inflow, exact solution
            u(1,2)=gauss_init(xLEFT(1)+z(1,2)/zx(1),t+a(k)*dt,...
                u(1,2),sig,2);
            u(2,Nlast(2))=2*u(2,N(2))-u(2,N(2)-1);                          % outflow, extrapolate
            u(2,NT(2))=2*u(2,Nlast(2))-u(2,N(2));
            U(1,1)=u(1,1)/zx(1);
            U(1,2)=u(1,2)/zx(1);
            U(2,Nlast(2))=u(2,Nlast(2))/zx(2);
            U(2,NT(2))=u(2,NT(2))/zx(2);
        elseif INIT == 4                                                    % update boundary values for Gaussian initial conditions
            u(2,NT(2))=gauss_init(xr,t+a(k)*dt,u(2,NT(2)),sig,NT(2));       % inflow, exact solution
            u(2,Nlast(2))=gauss_init(xLEFT(2)+z(2,Nlast(2))/zx(2),...
            t+a(k)*dt,u(2,Nlast(2)),sig,Nlast(2));
            u(1,2)=2*u(1,3)-u(1,4);                                         % outflow, extrapolate
            u(1,1)=2*u(1,2)-u(1,3);
            U(1,1)=u(1,1)/zx(1);
            U(1,2)=u(1,2)/zx(1);
            U(2,Nlast(2))=u(2,Nlast(2))/zx(2);
            U(2,NT(2))=u(2,NT(2))/zx(2);
        end

        for n=2:Nlast(1);                                                   % update, region 1
            zt(1,n)=-z(1,n)*zx(1)*w;
            u(1,n)=U(1,n)*zx(1);
        end
        for n=2:Nlast(2);                                                   % update, region 2
            zt(2,n)=(z(2,n)-1)*zx(2)*w;
            u(2,n)=U(2,n)*zx(2);
        end

    end                                                                     % end 3 stage RK "for" loop

    t=t+dt;                                                                 % increment time

    test=rem(kout,kplot);                                                   % save values for shock location plot every kplot steps
    if test == 0
        tout(kout)=t;
        xsout(kout)=xs;
    end

end                                                                         % end time stepping "while" loop
