function mac_non_conservative13
% MAC_NON_CONSERVATIVE13.M - solver based on MacCormack's predictor
% -corrector method
% A SHOCK FITTING PRIMER - CHAPTER 2, "Floating Shock-Fitting".
%
% Solves a non-conservative system of 2 PDE's
% dr/dt+(u+1)dr/dx=0, ds/dt+(u-1)ds/dt=0, r=u+g,     s=u-g.
%                                         u=(r+s)/2, g=(r-s)/2.
% computes r- and s-shocks.
%
% Copyright 2005-2009 M.D.Salas.

global kout eps0 dx dt dts FLAG IDETECT kplot tout xout
global xo_s I IS N NT CN kmax ICROSS kcross shock_type
global x u x_s w uL uR t tlast LAST Ro So
global g r s R S gL gR rL rR sL sR rLo rRo sRo sLo

ICROSS(1)=1/eps0;                                                           % set to a large number, used in function find_shk_xing
FLAG=0;                                                                     % trigger for shock crossing, set to 1 in function find_shk_xing
SAFETY=1000;                                                                % prevents over run, stops while loop

while LAST == 0                                                             % begin time stepping loop
   
    kout=kout+1;                                                            % step counter

    if kout > SAFETY                                                        % safety factor in case over out-of-control codition occurs
        LAST=1;
        disp('Stopped because possible out-of-control condition')
        disp('Check SAFETY value')
    end

    Amax=eps0;
    for n=1:NT                                                              % begin CFL condition
        Aabs=abs(u(n))+1;
        if Aabs > Amax
            Amax=Aabs;
        end
    end

    dt=CN*dx/Amax;                                                          % end CFL condition

    if kmax >= 2                                                            % check for shock crossing, reset dt if etest < 1
        find_shk_xing
    end


    if (t+dt) >= tlast                                                      % check that time does not exceed tlast
        dt=tlast-t;
        LAST=1;                                                             % trigger to stop while loop
    end

    for n=1:NT                                                              % store time interval, needed if shock crosses mesh point
        dts(n)=dt;
    end

    Ro=r;                                                                   % save latest values
    So=s;
    if kmax > 0
        xo_s=x_s;
        wo=w;
        sRo=sR;
        sLo=sL;
        rRo=rR;
        rLo=rL;
    end

    for j=1:2                                                               % j controls predictor-corrector loop
        np=2-j;
        nm=1-j;
                                                                            % in flow, n=1, out flow, n=N+1, b.c.s
        rx=(r(N+1)-r(N))/dx;                                                % assumes u >= 0 at left bondary
        rt=-(u(N+1)+1)*rx;
        if j == 1
            R(N+1)=r(N+1)+rt*dt;                                            % s is given at right boundary
        else                                                                % r computed at right boundary
            R(N+1)=.5*(Ro(N+1)+r(N+1)+rt*dt);
        end

        sx=(s(2)-s(1))/dx;
        st=-(u(1)-1)*sx;
        if j == 1
            S(1)=s(1)+st*dt;                                                % r is given at left boundary
        else                                                                % s computed at left boundary
            S(1)=.5*(So(1)+s(1)+st*dt);
        end

        for n=2:N                                                           % sweep through mesh points, except boundaries
            sigma=dts(n)/dx;
            rx=r(n+np)-r(n+nm);
            sx=s(n+np)-s(n+nm);

            if kmax > 0
                if j == 1 && I(n,1) ~= 0                                    % true if predictor stage & shock present in cell [n,n+1]
                    if I(n-1,1) == 0                                        % true if no shock in cell [n-1,n]
                        k=I(n,1);                                           % use shock index to right of n
                        eps1=(x_s(k)-x(n))/dx;
                        rx=((eps1-2)*r(n-1)+rL(k)+(1-eps1)*r(n))/2;
                        sx=((eps1-2)*s(n-1)+sL(k)+(1-eps1)*s(n))/2;
                    else                                                    % shock in cells [n-1,n] & [n,n+1]
                        k=I(n,1);
                        rx=(rL(k)-rR(k-1))*dx/(x_s(k)-x_s(k-1));
                        sx=(sL(k)-sR(k-1))*dx/(x_s(k)-x_s(k-1));
                    end
                elseif j == 2 && I(n-1,1) ~= 0                              % true if corrector stage & shock present in cell [n-1,n]
                    if I(n,1) == 0                                          % true if no shock in cell [n,n+1]
                        k=I(n-1,2);                                         % use shock index to left of n
                        eps1=(x(n)-x_s(k))/dx;
                        rx=-((eps1-2)*r(n+1)+rR(k)+(1-eps1)*r(n))/2;
                        sx=-((eps1-2)*s(n+1)+sR(k)+(1-eps1)*s(n))/2;
                    else                                                    % shock in cells [n-1,n] & [n,n+1]
                        k=I(n,1);
                        rx=(rL(k)-rR(k-1))*dx/(x_s(k)-x_s(k-1));
                        sx=(sL(k)-sR(k-1))*dx/(x_s(k)-x_s(k-1));
                    end
                end
            end

            rt=-(u(n)+1)*rx;                                                % evaluate time derivative
            st=-(u(n)-1)*sx;

            if j == 1
                R(n)=r(n)+sigma*rt;
                S(n)=s(n)+sigma*st;
            else
                R(n)=.5*(Ro(n)+r(n)+sigma*rt);
                S(n)=.5*(So(n)+s(n)+sigma*st);
            end
        end

        if kmax > 0
            for k=1:kmax                                                    % begin shock point computation
                n=IS(k);                                                    % set mesh point to left of k-th shock
                                                                            % compute left side
                if I(n,1) == k                                              % true if k-shock is first shock in cell [n,n+1]
                    if I(n-1,1) == 0 && I(n-2,1) == 0                       % true if no shock between cells [n-2,n]
                        epsL=(x_s(k)-x(n))/dx;
                        c1=2/(1+epsL);
                        rxL=(c1*(rL(k)-r(n-1))+.5*(r(n-2)-r(n)))/dx;
                        sxL=(c1*(sL(k)-s(n-1))+.5*(s(n-2)-s(n)))/dx;
                    elseif I(n-1,1) == 0 && I(n-2,1) ~= 0                   % true if shock in cell [n-2,n-1]
                        epsL=(x_s(k)-x(n))/dx;
                        c1=(2*epsL-1)/(1+epsL);
                        c2=3/(1+epsL);
                        rxL=(c1*r(n-1)+c2*rL(k)-2*r(n))/dx;
                        sxL=(c1*s(n-1)+c2*sL(k)-2*s(n))/dx;
                    elseif I(n-1,1) ~=  0                                   % true if shock in cell [n-1,n]
                        epsL=x_s(k)-x_s(k-1);
                        if epsL >= dx
                            rxL=(rL(k)-rR(k-1))/epsL;
                            sxL=(sL(k)-sR(k-1))/epsL;
                        else
                            rxL=0;
                            sxL=0;
                        end
                    end
                else                                                        % true if k-shock not first in cell [n,n+1]
                    rxL=0;                                                  % do not resolve cell microstructure
                    sxL=0;
                end
                                                                            % compute right side
                if I(n,2) == k                                              % true is k-shock is last shock in cell [n,n+1]
                    if I(n+1,1) == 0 && I(n+2,1) == 0                       % true if no shock between cells [n+1,n+3]
                        epsR=(x(n+1)-x_s(k))/dx;
                        c1=2/(1+epsR);
                        sxR=-(c1*(sR(k)-s(n+2))+.5*(s(n+3)-s(n+1)))/dx;
                        rxR=-(c1*(rR(k)-r(n+2))+.5*(r(n+3)-r(n+1)))/dx;
                    elseif I(n+1,1) == 0 && I(n+2,1) ~= 0                   % true if shock in cell [n+2,n+3]
                        epsR=(x(n+1)-x_s(k))/dx;
                        c1=(2*epsR-1)/(1+epsR);
                        c2=3/(1+epsR);
                        sxR=-(c1*s(n+2)+c2*sR(k)-2*s(n+1))/dx;
                        rxR=-(c1*r(n+2)+c2*rR(k)-2*r(n+1))/dx;
                    elseif I(n+1,1) ~= 0                                    % true if shock in cell [n+1,n+2]
                        epsR=x_s(k+1)-x_s(k);
                        if epsR >= dx
                            sxR=(sL(k+1)-sR(k))/epsR;
                            rxR=(rL(k+1)-rR(k))/epsR;
                        else
                            sxR=0;
                            rxR=0;
                        end
                    end
                else                                                        % true if k-shock not last shock in cell [n,n+1]
                    sxR=0;                                                  % do not resolve cell microstructure
                    rxR=0;
                end

                if shock_type(k) == 's'                                     % s-shock computation
                    rtL=-(uL(k)+1-w(k))*rxL;                                % compute rt on left side (on lambda+ characteristic)
                    rtR=rtL;                                                % R-H jump
                    wtr=.5*(rtL+rtR);
                    stL=-(uL(k)-1-w(k))*sxL;                                % compute st on left side (on lambda- chracteristic)
                    stR=-(uR(k)-1-w(k))*sxR;                                % compute st on right side (on lambda- chracteristic)
                    wts=.5*(stL+stR);

                    if j == 1
                        X_s(k)=xo_s(k)+w(k)*dt;                             % predictor for shock location
                        W(k)=wo(k)+.5*(wtr+wts)*dt;                         % predictor for shock speed
                        RL(k)=rLo(k)+rtL*dt;                                % compute r on left side (on lambda+ characteristic)
                        SR(k)=sRo(k)+stR*dt;                                % compute s on right side (on lambda- chracteristic)
                    else
                        X_s(k)=.5*(xo_s(k)+x_s(k)+w(k)*dt);                 % corrector for shock location
                        W(k)=.5*(wo(k)+w(k)+.5*(wtr+wts)*dt);               % corrector for shock speed
                        RL(k)=.5*(rLo(k)+rL(k)+rtL*dt);                     % compute r on left side (on lambda+ characteristic)
                        SR(k)=.5*(sRo(k)+sR(k)+stR*dt);                     % compute s on right side (on lambda- chracteristic)
                    end
                    RR(k)=RL(k);                                            % R-H jump: r constant across s-shock
                    SL(k)=4*(W(k)+1)-SR(k)-RL(k)-RR(k);                     % update s on left from R-H jump
                end

                if shock_type(k) == 'r'                                     % r-shock computation
                    rtR=-(uR(k)+1-w(k))*rxR;                                % compute rt on right side (on lambda+ characteristic)
                    stR=-(uR(k)-1-w(k))*sxR;                                % compute st on right side (on lambda- chracteristic)
                    stL=stR;                                                % R-H jump
                    rtL=-(uL(k)+1-w(k))*rxL;                                % compute rt on left side
                    wtr=.5*(rtL+rtR);
                    wts=.5*(stL+stR);

                    if j == 1
                        X_s(k)=xo_s(k)+w(k)*dt;                             % predictor for shock location
                        W(k)=wo(k)+.5*(wtr+wts)*dt;                         % predictor for shock speed
                        RL(k)=rLo(k)+rtL*dt;                                % compute r on left side (on lambda+ characteristic)
                        SR(k)=sRo(k)+stR*dt;                                % compute s on right side (on lambda- chracteristic)
                    else
                        X_s(k)=.5*(xo_s(k)+x_s(k)+w(k)*dt);                 % corrector for shock location
                        W(k)=.5*(wo(k)+w(k)+.5*(wtr+wts)*dt);               % corrector for shock speed
                        RL(k)=.5*(rLo(k)+rL(k)+rtL*dt);                     % compute r on left side (on lambda+ characteristic)
                        SR(k)=.5*(sRo(k)+sR(k)+stR*dt);                     % compute s on right side (on lambda- chracteristic)
                    end
                    SL(k)=SR(k);                                            % R-H jump: r constant across s-shock
                    RR(k)=4*(W(k)-1)-SR(k)-RL(k)-SL(k);                     % update r on right from R-H jump

                    if FLAG == 1                                            % if shock crossing taking place
                        x_s(kcross)=x_s(kcross-1);
                    end
                end                                                         % end shock point computation
            end
        end
         
        if kmax > 0
            x_s=X_s;
            w=W;
            rL=RL;
            sL=SL;
            rR=RR;
            sR=SR;
            uL=(rL+sL)/2;
            gL=(rL-sL)/2;
            uR=(rR+sR)/2;
            gR=(rR-sR)/2;
            mesh_xing                                                       % update mesh point crossed by a shock
        end

        r=R;                                                                % update u, g, r, s
        s=S;
        if kmax > 0
            for k=1:kmax-1
                if (IS(k)+1) == IS(k+1)                                     % if single mesh point lies between two shocks
                    ns=IS(k+1);                                             % update with shock averaged values
                    r(ns)=(rR(k)+rL(k+1))/2;
                    s(ns)=(sR(k)+sL(k+1))/2;
                end
            end
        end
        u=(r+s)/2;
        g=(r-s)/2;

    end                                                                     % end predictor-corrector loop

    if FLAG == 1                                                            % true if s-shock crossing occurs
        shock_cross
        FLAG=0;
    end

    t=t+dt;                                                                 % increment time
    
    if kmax > 0                                                             % check shocks for consistency
        remove_shock                                                        % if not consistent, remove shock
    end

    if IDETECT == 1                                                         % shock detection
        detect_shock
    end

    test=rem(kout,kplot);                                                   % save values for shock locations for plotting every kplot steps
    if test == 0 && kmax > 0
        for k=1:kmax
            tout(k,kout)=t;
            xout(k,kout)=x_s(k);
        end
    end    
        
end                                                                         % end time stepping "while" loop