% SHOCK_LAYER.M - Exact solution of Navier-Stokes equations for shock-layer
% A SHOCK FITTING PRIMER - CHAPTER 3.
%
% Copyright 2005-2009 M.D.Salas.

clear all
gamma=7/5;                                                                  % gamma=cp/cv
Re=1.;                                                                      % Re=Reynolds number
Pr=3/4;                                                                     % Pr=Prandtl number, solution valid for Pr=3/4 only
M=2;                                                                        % M=Mach number
nc=500;                                                                     % number of interval used in integration
nfig=1;

disp(' ')
disp('A SHOCK-FITTING PRIMER')
disp('Exact solution to Navier-Stokes equations for a shock layer')
disp(['Mach number= ' num2str(M)])
disp(['Reynolds number= ' num2str(Re)])
disp(['Prandtl number= ' num2str(Pr)])
disp(['gamma= ' num2str(gamma)])

gm=gamma-1;
gp=gamma+1;
sqrg=sqrt(gamma);
A=3*gp*Re/(8*gamma);
B=gamma/Re/Pr;
C=4*gm/(3*Re);
v1=1;                                                                       % b.c. at x-> -Inf
v2=gm/gp+2/gp/M^2;                                                          % b.c. at x-> +Inf
sqrv2=sqrt(v2);
dv=(v2-v1)/nc;

for n=1:nc+1
    v(n)=v1+(n-1)*dv;                                                       % v=specific volume
    vx=A*(v(n)-v1)*(v(n)-v2)/v(n);
    vxx=A*(v(n)^2-v2*v1)/v(n)^2;
    T(n)=1+gm*M^2*(1-v(n)^2)/2;                                             % T=temperature
    Tx=-gm*M^2*v(n)*vx;
    Txx=-gm*M^2*(vx+v(n)*vxx);
    ux=sqrg*M*vx;
    I1(n)=C*ux^2/T(n);
    I2(n)=B*Txx*vx/T(n);
    I3(n)=B*(Tx/T(n))^2;
    I4(n)=I2(n)-I3(n);
    I1I3(n)=C*sqrg*M*ux/T(n)-B*gm*M^2*v(n)*Tx/T(n)^2;
    sx(n)=(C*sqrg*M*ux+B*Txx)/T(n);                                         % sx=Sx/vx, sx is used for integration;
    Sx(n)=sx(n)*vx;                                                         % it is divided by vx because of change in variable
    Sx1(n)=I1(n)+I3(n)+I4(n);                                               % in integral, from dx to dv/(vx)
    rho(n)=1/v(n);
    u(n)=sqrg*M*v(n);
    p(n)=rho(n)*(1+gm*M^2*(1-v(n)^2)/2);
    S(n)=log(p(n))-gamma*log(rho(n));
    Mach(n)=u(n)/sqrt(gamma*T(n));
    x(n)=log((((sqrv2-v2)/(v(n)-v2))^v2)*((1-v(n))/(1-sqrv2)))/A/(1-v2);
end

sum(1)=0;
sum2(1)=0;
y1=sx(1);
z1=I1I3(1);
for n=1:nc
    y=sx(n+1);
    sum(n+1)=sum(n)+.5*(y+y1);
    y1=y;
    z=I1I3(n+1);
    sum2(n+1)=sum2(n)+.5*(z+z1);
    z1=z;
end
sum=sum*dv;
sum2=sum2*dv;

l=1;
nstep=nc/20;
for n=1:nstep:nc
    sum1(l)=sum(n);
    x1(l)=x(n);
    l=l+1;
end

figure(nfig)
fac=sqrt(pi*gamma/2);
M1=[1 2 5 10 25 100]; % M1(6)
ReL=[10^3 10^7]; % ReL(2)                                                   % Reynolds number based on L
for m=1:6 % M1 loop
    v2=gm/gp+2/gp/M1(m)^2;
    delta=8*gamma*(1+sqrt(v2))/(3*gp*M1(m)*fac)/(1-sqrt(v2));               % shock wave thickness
    M1m=M1(m);
    disp([' Mach= ' num2str(M1m) ', shock thickness= ' num2str(delta)])
end
Kn(1)=.01;                                                                 % Knudsen number
Kn(2)=Kn(1);
loglog(ReL,Kn,'k--')
hold on
for m=1:6
    Kn(1)=fac*M1(m)/ReL(1);
    Kn(2)=fac*M1(m)/ReL(2);
    loglog(ReL,Kn,'k-')
    hold on
end
xlabel('Reynolds number')
ylabel('Knudsen number')
title('Shock wave thickness')
hold off

nfig=nfig+1;
figure(nfig)
dm=.1;
for m=1:71
    Mp(m)=1+dm*(m-1);
    v2=gm/gp+2/gp/Mp(m)^2;
    deltap(m)=8*gamma*(1+sqrt(v2))/(3*gp*Mp(m)*fac)/(1-sqrt(v2));
    T2=1+.5*gm*(1-v2^2)*Mp(m)^2;
    rho2=1/v2;
    p2=rho2*T2;
    exps2=p2/rho2^gamma;
    S2(m)=log(exps2);                                                       % Entropy jump                                                       % Entropy peak
    S0(m)=S2(m)+log((rho2^(gp/2))/p2)+log((p2+1)/(rho2+1));                 % Entropy peak
end
semilogy(Mp,deltap,'k-')
xlabel('Mach number')
ylabel('Shock thickness')
title('Shock wave thickness')
grid on

nfig=nfig+1;
figure(nfig)
plot(Mp,S2,'k-',Mp,S0,'k--')
xlabel('Mach number')
ylabel('Entropy')
legend('S2', 'S0')
title('Entropy peak compared to entropy jump')

xsonic = interp1(Mach,x,1);                                                 % find location of sonic point
nfig=nfig+1;
figure(nfig)
plot(x,Sx,'k-',x,I1,'k--',x,I2,'r--',xsonic,0,'ro')
xlabel('x')
ylabel('Sx, I1, I2')
legend('Sx','I1','I2')
title('Entropy source terms: I1, I2, Sx is the sum of source terms')
grid on

nfig=nfig+1;
figure(nfig)
plot(x,Sx1,'k-',x,I1,'k--',x,I3,'r--',x,I4,'b-')
xlabel('x')
ylabel('Sx1, I1, I3, I4')
legend('Sx1','I1','I3','I4')
title('Entropy source terms: I1, I3, I4, Sx1 is the sum of source terms')
grid on

nfig=nfig+1;
figure(nfig)
plot(x,S,'k-',x1,sum1,'ko')
xlabel('x')
ylabel('S')
title('Entropy computed by integration of source terms and from p & rho')
legend('I1+I2','ln(p)-gln(rho)')
grid on

nfig=nfig+1;
figure(nfig)
plot(x,S,'k-',x,sum2,'r-')
xlabel('x')
ylabel('S')
title('Effect of neglecting I4 in entropy computation')
legend('S=I1+I3+I4','I1+I3')
grid on

nfig=nfig+1;
figure(nfig)
plot(x,Mach,'k-')
xlabel('x')
ylabel('Mach')
grid on

nfig=nfig+1;
figure(nfig)
plot(x,p,'k-')
xlabel('x')
ylabel('Pressure')
grid on

nfig=nfig+1;
figure(nfig)
plot(x,T,'k-')
xlabel('x')
ylabel('Temperature')
grid on

nfig=nfig+1;
figure(nfig)
plot(x,rho,'k-')
xlabel('x')
ylabel('Density')
grid on
