function lambda_gabutti_1
% LAMBDA_GABUTTI_1.M - solver based on Lambda Scheme, Gabutti variation
% Dependent variables R & Q
% A SHOCK FITTING PRIMER - CHAPTER 4, "Piston Driven Flows".
%
% Copyright 2005-2008 M.D.Salas.

global kout dx dt U t tlast LAST N NT CN a X Xt Xx R Q gm u
global uexact u_error x_error t_error

SAFETY=10000;                                                               % stops calculation if over run condition

while LAST == 0                                                             % begin time stepping loop

    kout=kout+1;                                                            % step counter

    if kout > SAFETY                                                        % stop calculation if out-of-control condition
        LAST=1;
        disp('Stopped because possible out-of-control condition')
        disp('Check SAFETY value in function mac_non_con_piston')
    end

    b=pistonlaw(t);
    bt=pistonlawt(t);
    c=exitboundary(t);
    ct=exitboundaryt(t);
    Xx=1/(c-b);

    Aabs=abs(U)+a*Xx;                                                       % begin CFL condition
    Amax=max(Aabs);
    dt=CN*dx/Amax;                                                          % end CFL condition

    if (t+dt) >= tlast                                                      % check that time does not exceed tlast
        dt=tlast-t;
        LAST=1;                                                             % trigger to stop while loop
    end

    sigma=dt/dx;

    for j=1:3                                                               % this is a three-steps scheme
        Xt=-Xx*(bt+X*(ct-bt));
        U=Xt+u*Xx;
        lambdaP=U+Xx*a;                                                     % characteristic slope in computational plane
        lambdaM=U-Xx*a;
        signLP=sign(lambdaP);
        signLM=sign(lambdaM);

        for n=2:N
            sLP=signLP(n);
            sLM=signLM(n);
            if j == 2
                if n == 2                        
                    Rx1=-(2*R(1)-3*R(2)+R(3));
                    R0=R(1)-Rx1;
                    Rx=(2*R(2)-3*R(1)+R0);
                    Qx1=-(2*Q(1)-3*Q(2)+Q(3));
                    Q0=Q(1)-Qx1;
                    Qx=(2*Q(2)-3*Q(1)+Q0);                        
                    if sLP < 0
                        Rx=-(2*R(n)-3*R(n+1)+R(n+2));
                    end
                    if sLM < 0
                        Qx=-(2*Q(n)-3*Q(n+1)+Q(n+2));
                    end
                elseif n == N
                    Rx=sLP*(R(n)-R(n-sLP));
                    Qx=sLM*(Q(n)-Q(n-sLM));
                    if sLP > 0
                        Rx=2*R(n)-3*R(n-1)+R(n-2);
                    end
                    if sLM > 0
                        Qx=2*Q(n)-3*Q(n-1)+Q(n-2); 
                    end
                else
                    Rx=sLP*(2*R(n)-3*R(n-sLP)+R(n-2*sLP));
                    Qx=sLM*(2*Q(n)-3*Q(n-sLM)+Q(n-2*sLM));
                end
            else
                Rx=sLP*(R(n)-R(n-sLP));
                Qx=sLM*(Q(n)-Q(n-sLM));
            end

            Rt=-lambdaP(n)*Rx*sigma;
            Qt=-lambdaM(n)*Qx*sigma;

            if j == 1
                Rn(n)=R(n)+Rt;
                Qn(n)=Q(n)+Qt;
                Ro(n)=R(n);
                Qo(n)=Q(n);
            elseif j == 2
                Rts(n)=Rt;
                Qts(n)=Qt;
            else
                Rn(n)=Ro(n)+.5*(Rts(n)+Rt);
                Qn(n)=Qo(n)+.5*(Qts(n)+Qt);
            end
        end

        % piston b.c.

        un1=pistonlawt(t+dt);
        if j == 1
            Qx=Q(2)-Q(1);
            Qt=-lambdaM(1)*Qx*sigma;
            Qn(1)=Q(1)+Qt;
            an1=gm*(Qn(1)+un1)/2;
            Rn(1)=2*an1/gm+un1;
            Qo(1)=Q(1);
        elseif j == 2
            Qx=-(2*Q(1)-3*Q(2)+Q(3));
            Qt=-lambdaM(1)*Qx*sigma;
            Qts(1)=Qt;
        else
            Qx=Q(2)-Q(1);
            Qt=-lambdaM(1)*Qx*sigma;
            Qn(1)=Qo(1)+.5*(Qts(1)+Qt);
            an1=gm*(Qn(1)+un1)/2;
            Rn(1)=2*an1/gm+un1;
        end


        % exit b.c. gas at rest
        if j == 1
            Rx=R(NT)-R(N);
            Rt=-lambdaP(NT)*Rx*sigma;
            Rn(NT)=R(NT)+Rt;
            Qn(NT)=Q(NT);
            Ro(NT)=R(NT);
        elseif j == 2
            Rx=2*R(NT)-3*R(NT-1)+R(NT-2);
            Rt=-lambdaP(NT)*Rx*sigma;
            Rts(NT)=Rt;
        else
            Rx=R(NT)-R(N);
            Rt=-lambdaP(NT)*Rx*sigma;
            Rn(NT)=Ro(NT)+.5*(Rts(NT)+Rt);
            Qn(NT)=Q(NT);
        end

                
        if j >= 2
            R=Rn;
            Q=Qn;
            a=gm*(R+Q)/4;
            u=(R-Q)/2;
        end
        
    end
    t=t+dt;

    exact
    b=pistonlaw(t);
    c=exitboundary(t);
    for n=1:NT
        u_error(n,kout)=u(n)-uexact(n);
        x_error(n,kout)=b+X(n)*(c-b);
        t_error(n,kout)=t;
    end

end                                                                         % end time stepping "while" loop

