function detect_shock
% DETECT_SHOCK.M - shock detection function
% A SHOCK FITTING PRIMER - CHAPTER 4, "Piston Driven Flows".
%
% Copyright 2005-2008 M.D.Salas.
%{
% DEFINITION OF SHOCK TYPE:
r-shock -> high pressure side on right side
l-shock -> high pressure side on left side
%}

global NT x inject_type N kmax IS t kout P R Q eps0 u dx 
global dx_det                                                               % dx_det defined in "initialpiston.m"

det_trigger=-.1;
                                                                            % dF(n)=diff(F)=F(n+1)-F(n), for n=1:N
du=abs(diff(u));
du(1)=0; du(2)=0; du(N)=0; du(NT)=0;

if kmax > 0
    for k=1:kmax
        nbar=IS(k);
        if nbar <= 2
            nbar=3;
        end
        for n=nbar-2:nbar+4                                                 % set du to zero in neighboor hood of existing shocks
        du(n+1)=0;                                                          % this avoids multiple detections of shocks due to wiggles
        end                                                                 % near new shocks
    end
end
   
[dumax,ns]=max(du);                                                         % find max dx/du
ns=ns+1;
if (ns > 3) && (ns < N-1)

    x1=x(ns-2);
    u1=u(ns-2);
    u2=u(ns-1);
    u3=u(ns);
    u4=u(ns+1);

    [aa,bb,cc,dd]=cubic_coeff(x1,u1,u2,u3,u4);                              % compute cubic coefficients

    det=bb^2-3*aa*cc;
    dQ=(Q(ns)-Q(ns-1))/dx;
    dR=(R(ns)-R(ns-1))/dx;

if (dR > -.21/dx_det) && (dQ < .21/dx_det)                                  % if true do not try to fit shock
    det=-1/eps0;
end
     

    
    if det > det_trigger
        u_shock=-bb/(3*aa);                                                 % u predicted by cubic fit
        x_shock=(((aa*u_shock+bb)*u_shock)+cc)*u_shock+dd;              	% shock location predicted by cubic fit           
        if (x(ns-2) < x_shock) && (x(ns+1) > x_shock)                       % if x_shock not in [x(ns-2), x(ns+1)] don't insert shock
            if P(ns-1) > P(ns+1)
                inject_type='l';                                            % du < 0 l-type shock
            else
                inject_type='r';                                            % du > 0 r-type shock
            end
            inj=inject_shock(ns,x_shock);
            if inj == 1
                shock_index
                print_shock
            end
        end
    end
end

function [a,b,c,d]=cubic_coeff(x1,u1,u2,u3,u4)
% CUBIC_COEFF.M - defines coefficients for cubic fit to determine shock
% formation. Maximum gradient of u located between u3=u(n) and u2=u(n-1),
% u1=u(n-2), u4=u(n+1), x1=x(n-2)
% Refer to Section 2.9 "Detection of Shock Formation"
%  A SHOCK FITTING PRIMER - CHAPTER 4
%
%  Copyright 2005-2008 M.D.Salas.

global dx

d1=u1*u2*(u3-u4)-((u1+u2)*u3-(u1+u2+u3-u4)*u4)*u4;
d2=u1*u2*(u3-u4)+((u1+u2)*u4+(u3-u1-u2-u4)*u3)*u3;
d3=u1*u3*(u4-u2)-((u1+u3)*u4-(u1+u3+u4-u2)*u2)*u2;
d4=u2*u3*(u1-u4)+((u2+u3)*u4+(u1-u4-u3-u2)*u1)*u1;

e1=x1/d4;
e2=(x1+dx)/d3;
e3=(x1+2*dx)/d2;
e4=(x1+3*dx)/d1;

a=e1-e2+e3-e4;                                                              % coefficients of x=a*u^3+b*u^2+c*u+d
b=-(u2+u3+u4)*e1+(u1+u3+u4)*e2-(u1+u2+u4)*e3+(u1+u2+u3)*e4;
c=(u2*(u3+u4)+u4*u3)*e1-(u1*(u3+u4)+u4*u3)*e2+...
    (u1*(u2+u4)+u2*u4)*e3-(u1*(u2+u3)+u2*u3)*e4;
d=(u1*e2-u2*e1)*u3*u4+(u3*e4-u4*e3)*u1*u2;

function inj=inject_shock(ns,x_shock)
% INJECT_SHOCK.M - shock injection function
% initializes newly detected shock
% A SHOCK FITTING PRIMER - CHAPTER 4, "Piston Driven Flows".
%
% Copyright 2005-2008 M.D.Salas.

global kout IS kmax x_s w shock_type inject_type x S gamma t
global uL uR PL PR SL SR ZL ZR aL aR RR RL QR QL Q R

xn=(x(ns)+x(ns-1))/2.;                                                      % inject shock midpoint where max gradient was found
gm=gamma-1;
gp=gamma+1.;    
d1=1.5;
d2=.5;

print_event
disp(['shock formation detected at t= ' num2str(t) ', x_shock= ' num2str(x_shock) ])
disp(['at step = ' num2str(kout) ', between n [' num2str(ns-1)...
    ',' num2str(ns) ']'])

if inject_type == 'l'                                                       % HIGH pressure side on left 
    Q2=d1*Q(ns-1)-d2*Q(ns-2);
    S2=d1*S(ns-1)-d2*S(ns-2);
    R2=d1*R(ns-1)-d2*R(ns-2);
    u2=(R2-Q2)/2;
    a2=gm*(R2+Q2)/4;
    Z2=a2^2/gamma;
    P2=(gamma*log(Z2)-S2)/gm;
    p2=exp(P2);    
    rho2=p2/Z2;
    
    Q1=d1*Q(ns)-d2*Q(ns+1);
    R1=d1*R(ns)-d2*R(ns+1);
    S1=d1*S(ns)-d2*S(ns+1);
    u1=(R1-Q1)/2;
    a1=gm*(R1+Q1)/4;
    Z1=a1^2/gamma;
    P1=(gamma*log(Z1)-S1)/gm;
    p1=exp(P1);
    sign0=1;  
else                                                                        % HIGH pressure side on right
    Q2=d1*Q(ns)-d2*Q(ns+1);
    S2=d1*S(ns)-d2*S(ns+1);
    R2=d1*R(ns)-d2*R(ns+1);
    u2=(R2-Q2)/2;
    a2=gm*(R2+Q2)/4;
    Z2=a2^2/gamma;
    P2=(gamma*log(Z2)-S2)/gm;
    p2=exp(P2);
    rho2=p2/Z2;
    
    R1=d1*R(ns-1)-d2*R(ns-2);
    Q1=d1*Q(ns-1)-d2*Q(ns-2);
    S1=d1*S(ns-1)-d2*S(ns-2);
    u1=(R1-Q1)/2;
    a1=gm*(R1+Q1)/4;
    Z1=a1^2/gamma;
    P1=(gamma*log(Z1)-S1)/gm;
    p1=exp(P1);
    sign0=-1;   
end

rho1=p1/Z1;
SM1rel=2*(rho2/rho1)/(gp-gm*(rho2/rho1));
if SM1rel < 1
    inj=0;
    disp('shock Mach number less than 1; shock not inserted.') 
   return
end
inj=1;
wn=u1+sign0*sqrt(SM1rel)*a1;
if inject_type == 'l'
    disp('l-shock inserted')
else
    disp('r-shock inserted ')
end

if kmax == 0                                                                % no other shocks present
    kmax=1;
    x_s(1)=xn;
    w(1)=wn;
    shock_type(1)=inject_type;

    if inject_type == 'l'
        uL(1)=u2;
        uR(1)=u1;
        PL(1)=P2;
        PR(1)=P1;
        SL(1)=S2;
        SR(1)=S1;
        ZL(1)=Z2;
        ZR(1)=Z1;
        aL(1)=a2;
        aR(1)=a1;
        RL(1)=R2;
        RR(1)=R1;
        QL(1)=Q2;
        QR(1)=Q1;
    else 
        uL(1)=u1;
        uR(1)=u2;
        PL(1)=P1;
        PR(1)=P2;
        SL(1)=S1;
        SR(1)=S2;
        ZL(1)=Z1;
        ZR(1)=Z2;
        aL(1)=a1;
        aR(1)=a2;
        RL(1)=R1;
        RR(1)=R2;
        QL(1)=Q1;
        QR(1)=Q2;
    end
else                                                                        % other shocks presents
    nkmax=IS(kmax);
    n1=IS(1);
    if nkmax < ns                                                           % last shock is located to the left of new shock
        kmax=kmax+1;
        x_s(kmax)=xn;
        w(kmax)=wn;
        shock_type(kmax)=inject_type;

        if inject_type == 'l'
            uL(kmax)=u2;
            uR(kmax)=u1;
            PL(kmax)=P2;
            PR(kmax)=P1;
            SL(kmax)=S2;
            SR(kmax)=S1;
            ZL(kmax)=Z2;
            ZR(kmax)=Z1;
            aL(kmax)=a2;
            aR(kmax)=a1;
            RL(kmax)=R2;
            RR(kmax)=R1;
            QL(kmax)=Q2;
            QR(kmax)=Q1;
        else
            uL(kmax)=u1;
            uR(kmax)=u2;
            PL(kmax)=P1;
            PR(kmax)=P2;
            SL(kmax)=S1;
            SR(kmax)=S2;
            ZL(kmax)=Z1;
            ZR(kmax)=Z2;
            aL(kmax)=a1;
            aR(kmax)=a2;
            RL(kmax)=R1;
            RR(kmax)=R2;
            QL(kmax)=Q1;
            QR(kmax)=Q2;
        end
    elseif n1 > ns                                                          % first shock located to the right of new shock
        kmax=kmax+1;
        for k=2:kmax
            x_s(k)=x_s(k-1);
            w(k)=w(k-1);
            uL(k)=uL(k-1);
            uR(k)=uR(k-1);
            PL(k)=PL(k-1);
            PR(k)=PR(k-1);
            SL(k)=SL(k-1);
            SR(k)=SR(k-1);
            ZL(k)=ZL(k-1);
            ZR(k)=ZR(k-1);
            aL(k)=aL(k-1);
            aR(k)=aR(k-1);
            QR(k)=QR(k-1);
            QL(k)=QL(k-1);
            RR(k)=RR(k-1);
            RL(k)=RL(k-1);
            shock_type(k)=shock_type(k-1);
        end
        x_s(1)=xn;
        w(1)=wn;
        shock_type(1)=inject_type;

        if inject_type == 'l'
            uL(1)=u2;
            uR(1)=u1;
            PL(1)=P2;
            PR(1)=P1;
            SL(1)=S2;
            SR(1)=S1;
            ZL(1)=Z2;
            ZR(1)=Z1;
            aL(1)=a2;
            aR(1)=a1;
            RL(1)=R2;
            RR(1)=R1;
            QL(1)=Q2;
            QR(1)=Q1;
        else
            uL(1)=u1;
            uR(1)=u2;
            PL(1)=P1;
            PR(1)=P2;
            SL(1)=S1;
            SR(1)=S2;
            ZL(1)=Z1;
            ZR(1)=Z2;
            aL(1)=a1;
            aR(1)=a2;
            RL(1)=R1;
            RR(1)=R2;
            QL(1)=Q1;
            QR(1)=Q2;
        end
    else                                                                    % new shock in between other shocks
        for k=1:kmax                                                        % find shock index to left of new shock
            n=IS(k);
            if n < ns
                kleft=k;
            end
        end
        for k=kleft+1:kmax
            x_s(k+1)=x_s(k);
            w(k+1)=w(k);
            uL(k+1)=uL(k);
            uR(k+1)=uR(k);
            PL(k+1)=PL(k);
            PR(k+1)=PR(k);
            SL(k+1)=SL(k);
            SR(k+1)=SR(k);
            ZL(k+1)=ZL(k);
            ZR(k+1)=ZR(k);
            aL(k+1)=aL(k);
            aR(k+1)=aR(k);
            QR(k+1)=QR(k);
            QL(k+1)=QL(k);
            RR(k+1)=RR(k);
            RL(k+1)=RL(k);
            shock_type(k+1)=shock_type(k);
        end
        kmax=kmax+1;
        x_s(kleft+1)=xn;
        w(kleft+1)=wn;
        shock_type(kleft+1)=inject_type;
        if inject_type == 'l'
            uL(kleft+1)=u2;
            uR(kleft+1)=u1;
            PL(kleft+1)=P2;
            PR(kleft+1)=P1;
            SL(kleft+1)=S2;
            SR(kleft+1)=S1;
            ZL(kleft+1)=Z2;
            ZR(kleft+1)=Z1;
            aL(kleft+1)=a2;
            aR(kleft+1)=a1;
            RL(kleft+1)=R2;
            RR(kleft+1)=R1;
            QL(kleft+1)=Q2;
            QR(kleft+1)=Q1;
        else
            uL(kleft+1)=u1;
            uR(kleft+1)=u2;
            PL(kleft+1)=P1;
            PR(kleft+1)=P2;
            SL(kleft+1)=S1;
            SR(kleft+1)=S2;
            ZL(kleft+1)=Z1;
            ZR(kleft+1)=Z2;
            aL(kleft+1)=a1;
            aR(kleft+1)=a2;
            RL(kleft+1)=R1;
            RR(kleft+1)=R2;
            QL(kleft+1)=Q1;
            QR(kleft+1)=Q2;
        end
    end
end




