function initialpiston
% INITIALPISTON.M - defines initial conditions
% A SHOCK FITTING PRIMER - CHAPTER 4, "Piston Driven Flows"
%
% Copyright 2005-2008 M.D.Salas.

global eps0 dx N NT nfig sg x X t INIT kmax_save Area_throat x_throat
global a u P S Z gamma Xx Xt U kmax x_s w shock_type LAx pexit_final
global uL uR PL PR SL SR ZL ZR aL aR I R Q RR RL QR QL area_index IS LAt
global dx_det gm gp

%{
% l-shock = shock with high pressure on left side
% r-shock = shock with high pressure on right side
%           UNIT PROBLEMS
% INIT= 1 b(t)=t^3, c(t)=.05+sqrt(gamma)*t
% INIT= 2 single shock moving right
% INIT= 3 single shock moving left
% INIT= 4 b(t)=0.95-sqrt(gamma)*t, c(t)=1-t^3
% INIT= 5 two shocks colliding
% INIT= 6 two l-shocks coalescing
% INIT= 7 two r-shocks coalescing
% INIT= 8 l-shock interacts with contact
% INIT= 9 r-shock interacts with contact
% INIT=10 l-shock reflection from wall
% INIT=11 r-shock reflection from wall
%           MORE COMPLEX PROBLEMS
% INIT=12 shock detection and reflection; b(t)=1-cos(pi*t/2), c(t)=2
% INIT=13 mirror of case 12
% INIT=14 cases 12 and 13 combined
% INIT=15 asymmetric version of case 14
% INIT=16 another asymmetric version of case 14
%}

t=0;                                                                        % time
gamma=1.4;                                                                  % gamma=cp/cv
gm=gamma-1;
gp=gamma+1;
sg=sqrt(gamma);
kmax_save=0;                                                                % counter used to plot shock paths
NT=N+1;                                                                     % total number of mesh points
eps0=10^-10;                                                                % small number
nfig=0;                                                                     % figure counter
dx=1/N;                                                                     % mesh interval size
dx_det=max([.01,dx]);
dx_det=min([.02,dx_det]);                                                   % used in shock detection
x=zeros(1,NT);
X=zeros(1,NT);
u=zeros(1,NT);
P=zeros(1,NT);
S=zeros(1,NT);
a=zeros(1,NT);
Z=zeros(1,NT);
U=zeros(1,NT);
LAx=zeros(1,NT);
LAt=zeros(1,NT);
I(:,1)=zeros(NT,1);                                                         % indexes to track shock presence in interval between mesh points [n,n+1]
I(:,2)=zeros(NT,1);                                                         % I(n,1) is the lowest shock k index in interval, I(n,2) is the highest k index

switch (INIT)                                                               % define initial conditions according to case value INIT
    case {1, 4, 12, 13, 14, 15, 16}                                         % Piston law b(t)=t^3 or c(t)=1-t^3
        kmax=0;

        b=pistonlaw(t);                                                     % call piston routine
        c=exitboundary(t);                                                  % call exit boundary routine
        Xx=1/(c-b);
        
        gas_at_rest
        
    case {2, 10}                                                            % single shock moving right
        kmax=1;
        shock_type(1)='l';                                                  % high pressure side on left

        b=pistonlaw(t);                                                     % call piston routine
        c=exitboundary(t);                                                  % call exit boundary routine
        Xx=1/(c-b);
        
        SM1rel=2;                                                           % Mach relative to shock squared
        u1=0;                                                               % values to right of shock
        a1=sg;
        p1=1;
        rho1=1;
        sign0=1;
        [p2,rho2,Z2,S2,a2,u2,wn]=rankine_hugoniot(SM1rel,u1,a1,p1,rho1,sign0);
        w(1)=wn;
        
        gas_at_rest
        
        ns=int32(NT/3);
        x_s(1)=(x(ns)+x(ns+1))/2;                                           % shock location set at about 1/3 of the total interval

        uL(1)=u2;
        uR(1)=0;
        PL(1)=log(p2);
        PR(1)=0;
        SL(1)=S2;
        SR(1)=0;
        ZL(1)=Z2;
        ZR(1)=1;
        aL(1)=a2;
        aR(1)=sg;
        for n=1:NT
            if x(n) < x_s(1)
                u(n)=u2;
                P(n)=PL(1);
                S(n)=S2;
                Z(n)=Z2;
                a(n)=a2;
                U(n)=Xt(n)+u(n)*Xx;
            end
        end
        
        shock_index

    case {3, 11}                                                            % single shock moving left
        kmax=1;
        shock_type(1)='r';                                                  % high pressure side on right

        b=pistonlaw(t);                                                     % call piston routine
        c=exitboundary(t);                                                  % call exit boundary routine
        Xx=1/(c-b);
        
        SM1rel=2;                                                           % Mach relative to shock squared
        u1=0;                                                               % values to left of shock
        a1=sg;
        p1=1;
        rho1=1;
        sign0=-1;
        [p2,rho2,Z2,S2,a2,u2,wn]=rankine_hugoniot(SM1rel,u1,a1,p1,rho1,sign0);
        w(1)=wn;
        
        gas_at_rest
        
        ns=2*int32(NT/3);
        x_s(1)=(x(ns)+x(ns+1))/2;                                           % shock location set at about 2/3 of the total interval

        uR(1)=u2;
        uL(1)=0;
        PR(1)=log(p2);
        PL(1)=0;
        SR(1)=S2;
        SL(1)=0;
        ZR(1)=Z2;
        ZL(1)=1;
        aR(1)=a2;
        aL(1)=sg;
        for n=1:NT
            if x(n) > x_s(1)
                u(n)=u2;
                P(n)=PR(1);
                S(n)=S2;
                Z(n)=Z2;
                a(n)=a2;
                U(n)=Xt(n)+u(n)*Xx;
            end
        end

        shock_index

    case 5                                                                  % two shocks colliding
        kmax=2;                                                             % max shock index set to 2
        shock_type(1)='l';                                                  % high pressure side on left

        b=pistonlaw(t);                                                     % call piston routine
        c=exitboundary(t);                                                  % call exit boundary routine
        Xx=1/(c-b);
        
        SM1rel=2;                                                           % MACH RELATIVE TO SHOCK SQUARED
        u1=0;
        a1=sg;
        p1=1;
        rho1=1;
        sign0=1;
        [p2,rho2,Z2,S2,a2,u2,wn]=rankine_hugoniot(SM1rel,u1,a1,p1,rho1,sign0);
        w(1)=wn;
        
        gas_at_rest
        
        ns=int32(NT/3);
        x_s(1)=(x(ns)+x(ns+1))/2;                                           % shock location set at about 1/3 of the interval

        uL(1)=u2;
        uR(1)=0;
        PL(1)=log(p2);
        PR(1)=0;
        SL(1)=S2;
        SR(1)=0;
        ZL(1)=Z2;
        ZR(1)=1;
        aL(1)=a2;
        aR(1)=sg;
        for n=1:NT
            if x(n) < x_s(1)
                u(n)=u2;
                P(n)=PL(1);
                S(n)=S2;
                Z(n)=Z2;
                a(n)=a2;
                U(n)=Xt(n)+u(n)*Xx;
            end
        end

        shock_type(2)='r';                                                  % high pressure side on right

        SM1rel=3;                                                           % MACH RELATIVE TO SHOCK SQUARED
        u1=0;
        a1=sg;
        p1=1;
        rho1=1;
        sign0=-1;
        [p2,rho2,Z2,S2,a2,u2,wn]=rankine_hugoniot(SM1rel,u1,a1,p1,rho1,sign0);
        w(2)=wn;

        ns=2*int32(NT/3);
        x_s(2)=(x(ns)+x(ns+1))/2;                                           % shock location set at about 2/3 of the interval

        uR(2)=u2;
        uL(2)=0;
        PR(2)=log(p2);
        PL(2)=0;
        SR(2)=S2;
        SL(2)=0;
        ZR(2)=Z2;
        ZL(2)=1;
        aR(2)=a2;
        aL(2)=sg;
        for n=1:NT
            if x(n) > x_s(2)
                u(n)=u2;
                P(n)=PR(2);
                S(n)=S2;
                Z(n)=Z2;
                a(n)=a2;
                U(n)=Xt(n)+u(n)*Xx;
            end
        end

        shock_index

    case 6                                                                  % two l-shocks coalescing
        kmax=2;
        shock_type(2)='l';                                                  % high pressure side on left

        b=pistonlaw(t);                                                     % call piston routine
        c=exitboundary(t);                                                  % call exit boundary routine
        Xx=1/(c-b);
        
        SM1rel=2;                                                           % MACH RELATIVE TO SHOCK SQUARED
        u1=0;
        a1=sg;
        p1=1;
        rho1=1;
        sign0=1;
        [p2,rho2,Z2,S2,a2,u2,wn]=rankine_hugoniot(SM1rel,u1,a1,p1,rho1,sign0);     
        w(2)=wn;
            
        gas_at_rest
        
        ns=int32(NT/3);
        x_s(2)=(x(ns)+x(ns+1))/2;                                           % shock location set at about 1/3 of the total interval

        uL(2)=u2;
        uR(2)=0;
        PL(2)=log(p2);
        PR(2)=0;
        SL(2)=S2;
        SR(2)=0;
        ZL(2)=Z2;
        ZR(2)=1;
        aL(2)=a2;
        aR(2)=sg;
        for n=1:NT
            if x(n) < x_s(2)
                u(n)=u2;
                P(n)=PL(2);
                S(n)=S2;
                Z(n)=Z2;
                a(n)=a2;
                U(n)=Xt(n)+u(n)*Xx;
            end
        end

        shock_type(1)='l';                                                  % high pressure side on left

        SM2rel=2;                                                           % MACH RELATIVE TO SHOCK SQUARED
        sign0=1;
        [p3,rho3,Z3,S3,a3,u3,wn]=rankine_hugoniot(SM2rel,u2,a2,p2,rho2,sign0);
        w(1)=wn;
        
        ns=int32(NT/4);
        x_s(1)=(x(ns)+x(ns+1))/2;                                           % shock location set at about 2/3 of the total interval

        uR(1)=u2;
        uL(1)=u3;
        PR(1)=log(p2);
        PL(1)=log(p3);
        SR(1)=S2;
        SL(1)=S3;
        ZR(1)=Z2;
        ZL(1)=Z3;
        aR(1)=a2;
        aL(1)=a3;
        for n=1:NT
            if x(n) < x_s(1)
                u(n)=u3;
                P(n)=PL(1);
                S(n)=S3;
                Z(n)=Z3;
                a(n)=a3;
                U(n)=Xt(n)+u(n)*Xx;
            end
        end

        shock_index

    case 7                                                                  % two l-shocks coalescing
        kmax=2;
        shock_type(1)='r';                                                  % high pressure side on right

        b=pistonlaw(t);                                                     % call piston routine
        c=exitboundary(t);                                                  % call exit boundary routine
        Xx=1/(c-b);
        
        SM1rel=2;                                                           % MACH RELATIVE TO SHOCK SQUARED
        u1=0;
        a1=sg;
        p1=1;
        rho1=1;
        sign0=-1;
        [p2,rho2,Z2,S2,a2,u2,wn]=rankine_hugoniot(SM1rel,u1,a1,p1,rho1,sign0);
        w(1)=wn;
            
        gas_at_rest
        
        ns=2*int32(NT/3);
        x_s(1)=(x(ns)+x(ns+1))/2;                                           % shock location set at about 2/3 of the total interval

        uL(1)=0;
        uR(1)=u2;
        PL(1)=0;
        PR(1)=log(p2);
        SL(1)=0;
        SR(1)=S2;
        ZL(1)=1;
        ZR(1)=Z2;
        aL(1)=sg;
        aR(1)=a2;
        for n=1:NT
            if x(n) > x_s(1)
                u(n)=u2;
                P(n)=PR(1);
                S(n)=S2;
                Z(n)=Z2;
                a(n)=a2;
                U(n)=Xt(n)+u(n)*Xx;
            end
        end

        shock_type(2)='r';                                                  % high pressure side on right

        SM2rel=2;                                                           % MACH RELATIVE TO SHOCK SQUARED
        sign0=-1;
        [p3,rho3,Z3,S3,a3,u3,wn]=rankine_hugoniot(SM2rel,u2,a2,p2,rho2,sign0);
        w(2)=wn;

        ns=3*int32(NT/4);
        x_s(2)=(x(ns)+x(ns+1))/2;                                           % shock location set at about 3/4 of the total interval

        uR(2)=u3;
        uL(2)=u2;
        PR(2)=log(p3);
        PL(2)=log(p2);
        SR(2)=S3;
        SL(2)=S2;
        ZR(2)=Z3;
        ZL(2)=Z2;
        aR(2)=a3;
        aL(2)=a2;
        for n=1:NT
            if x(n) > x_s(2)
                u(n)=u3;
                P(n)=PR(2);
                S(n)=S3;
                Z(n)=Z3;
                a(n)=a3;
                U(n)=Xt(n)+u(n)*Xx;
            end
        end

        shock_index
        
    case {8, 8.5}
        kmax=2;
        shock_type(1)='l';                                                  % high pressure side on left

        b=pistonlaw(t);                                                     % call piston routine
        c=exitboundary(t);                                                  % call exit boundary routine
        Xx=1/(c-b);
        
        SM1rel=2;                                                           % Mach relative to shock squared
        u1=0;
        a1=sg;
        p1=1;
        rho1=1;
        sign0=1;
        [p2,rho2,Z2,S2,a2,u2,wn]=rankine_hugoniot(SM1rel,u1,a1,p1,rho1,sign0);
        w(1)=wn;

        gas_at_rest
        
        ns=int32(NT/3);
        x_s(1)=(x(ns)+x(ns+1))/2;                                           % shock location set at about 1/3 of the total interval
        
        uL(1)=u2;
        uR(1)=0;
        PL(1)=log(p2);
        PR(1)=0;
        SL(1)=S2;
        SR(1)=0;
        ZL(1)=Z2;
        ZR(1)=1;
        aL(1)=a2;
        aR(1)=sg;
        for n=1:NT
            if x(n) < x_s(1)
                u(n)=u2;
                P(n)=PL(1);
                S(n)=S2;
                Z(n)=Z2;
                a(n)=a2;
                U(n)=Xt(n)+u(n)*Xx;
            end
        end
        
        ns=2*int32(NT/3);
        x_s(2)=(x(ns)+x(ns+1))/2;
        w(2)=0;
        shock_type(2)='c';
        
        Z3=1/2;
        p3=1;
        if INIT == 8.5
            Z3=1.5;
        end
        a3=sqrt(gamma*Z3);
        rho3=p3/Z3;
        S3=-gamma*log(rho3);
        for n=1:NT
            if x(n) > x_s(2)
                Z(n)=Z3;
                a(n)=a3;
                S(n)=S3;
            end
        end
        uL(2)=0;
        uR(2)=0;
        PL(2)=0;
        PR(2)=0;
        SL(2)=0;
        SR(2)=S3;
        ZL(2)=1;
        ZR(2)=Z3;
        aL(2)=sg;
        aR(2)=a3;

        shock_index 
        
    case {9, 9.5}
        kmax=2;
        shock_type(2)='r';                                                  % high pressure side on right

        b=pistonlaw(t);                                                     % call piston routine
        c=exitboundary(t);                                                  % call exit boundary routine
        Xx=1/(c-b);
        
        SM1rel=2;                                                           % Mach relative to shock squared
        u1=0;
        a1=sg;
        p1=1;
        rho1=1;
        sign0=-1;
        [p2,rho2,Z2,S2,a2,u2,wn]=rankine_hugoniot(SM1rel,u1,a1,p1,rho1,sign0);
        w(2)=wn;

        gas_at_rest
        
        ns=2*int32(NT/3);
        x_s(2)=(x(ns)+x(ns+1))/2;                                           % shock location set at about 2/3 of the total interval

        uR(2)=u2;
        uL(2)=0;
        PR(2)=log(p2);
        PL(2)=0;
        SR(2)=S2;
        SL(2)=0;
        ZR(2)=Z2;
        ZL(2)=1;
        aR(2)=a2;
        aL(2)=sg;
        for n=1:NT
            if x(n) > x_s(2)
                u(n)=u2;
                P(n)=PR(2);
                S(n)=S2;
                Z(n)=Z2;
                a(n)=a2;
                U(n)=Xt(n)+u(n)*Xx;
            end
        end
        
        ns=int32(NT/3);
        x_s(1)=(x(ns)+x(ns+1))/2;
        w(1)=0;
        shock_type(1)='c';
        
        Z3=1/2;
        p3=1;
        if INIT == 9.5
            Z3=1.5;
        end
        a3=sqrt(gamma*Z3);
        rho3=p3/Z3;
        S3=-gamma*log(rho3);
        for n=1:NT
            if x(n) < x_s(1)
                Z(n)=Z3;
                a(n)=a3;
                S(n)=S3;
            end
        end
        uL(1)=0;
        uR(1)=0;
        PL(1)=0;
        PR(1)=0;
        SL(1)=S3;
        SR(1)=0.;
        ZL(1)=Z3;
        ZR(1)=1;
        aL(1)=a3;
        aR(1)=sg;
        
        shock_index
        
    case 17
        kmax=1;
        shock_type(1)='l';                                                  % high pressure side on left

        b=pistonlaw(t);                                                     % call piston routine
        c=exitboundary(t);                                                  % call exit boundary routine
        Xx=1/(c-b);

        SM1rel=12.25;                                                       % Mach relative to shock squared
        u1=0;                                                               % values to right of shock
        a1=sg;
        p1=1;
        rho1=1;
        sign0=1;
        [p2,rho2,Z2,S2,a2,u2,wn]=rankine_hugoniot(SM1rel,u1,a1,p1,rho1,sign0);
        w(1)=wn;

        gas_at_rest

        for n=1:NT
            area=area_f(x(n), t, area_index);
            LAx(n)=areax_f(x(n),t, area_index)/area;                        % define area term
            LAt(n)=areat_f(x(n),t, area_index)/area;
        end
        
        ns=15;
        x_s(1)=(x(ns)+x(ns+1))/2;                                           % shock location set at interval [15,16]

        uL(1)=u2;
        uR(1)=0;
        PL(1)=log(p2);
        PR(1)=0;
        SL(1)=S2;
        SR(1)=0;
        ZL(1)=Z2;
        ZR(1)=1;
        aL(1)=a2;
        aR(1)=sg;
        for n=1:NT
            if x(n) < x_s(1)
                u(n)=u2;
                P(n)=PL(1);
                S(n)=S2;
                Z(n)=Z2;
                a(n)=a2;
                U(n)=Xt(n)+u(n)*Xx;
            end
        end

        shock_index
        
        case 18
        kmax=1;
        shock_type(1)='l';                                                  % high pressure side on left

        b=pistonlaw(t);                                                     % call piston routine
        c=exitboundary(t);                                                  % call exit boundary routine
        Xx=1/(c-b);

        SM1rel=2;                                                           % Mach relative to shock squared
        u1=0;                                                               % values to right of shock
        a1=sg;
        p1=1;
        rho1=1;
        sign0=1;
        [p2,rho2,Z2,S2,a2,u2,wn]=rankine_hugoniot(SM1rel,u1,a1,p1,rho1,sign0);
        w(1)=wn;

        gas_at_rest

        for n=1:NT
            area=area_f(x(n), t, area_index);
            LAx(n)=areax_f(x(n), t, area_index)/area;                       % define area term
            LAt(n)=areat_f(x(n), t, area_index)/area;
        end
        
        ns=15;
        x_s(1)=(x(ns)+x(ns+1))/2;                                           % shock location set at interval [15,16]

        uL(1)=u2;
        uR(1)=0;
        PL(1)=log(p2);
        PR(1)=0;
        SL(1)=S2;
        SR(1)=0;
        ZL(1)=Z2;
        ZR(1)=1;
        aL(1)=a2;
        aR(1)=sg;
        for n=1:NT
            if x(n) < x_s(1)
                u(n)=u2;
                P(n)=PL(1);
                S(n)=S2;
                Z(n)=Z2;
                a(n)=a2;
                U(n)=Xt(n)+u(n)*Xx;
            end
        end

        shock_index
        
    case 19                                                                 % steady Laval nozzle, no shocks
        Area_throat=2;                                                      % used in function exact_Q1D
        x_throat=1;                                                         % used in function exact_Q1D
        kmax=0;
        
        b=pistonlaw(t);                                                     % call piston routine
        c=exitboundary(t);                                                  % call exit boundary routine
        bt=pistonlawt(t);
        ct=exitboundaryt(t);
        Xx=1/(c-b);
        for n=1:NT
            X(n)=dx*(n-1);                                                  % computational coordinate                                             % contravariant velocity
        end
        Xt=-Xx*(bt+X*(ct-bt));                                              % derivative of X wrt t
        x=b+X/Xx;                                                           % physical coordinate
        
        exact_Q1D
        
        U=Xt+u*Xx; 
        for n=1:NT
            area=area_f(x(n), t, area_index);
            LAx(n)=areax_f(x(n), t, area_index)/area;                       % define area term
            LAt(n)=areat_f(x(n), t, area_index)/area;
        end

    case {20, 22, 23, 24, 25, 26}                                           % steady Laval nozzle, with shock
        Area_throat=2;                                                      % used in exact_Q1D_s
        x_throat=1;                                                         % used in exact_Q1D_s

        pexit_final=0.7735;                                                 % b.c. at exit plane corresponding to shock at x=1.5
        kmax=1;

        b=pistonlaw(t);                                                     % call piston routine
        c=exitboundary(t);                                                  % call exit boundary routine
        bt=pistonlawt(t);
        ct=exitboundaryt(t);
        Xx=1/(c-b);
        for n=1:NT
            X(n)=dx*(n-1);                                                  % computational coordinate                                             % contravariant velocity
        end
        Xt=-Xx*(bt+X*(ct-bt));                                              % derivative of X wrt t
        x=b+X/Xx;                                                           % physical coordinate
        
        exact_Q1D_s

        U=Xt+u*Xx; 
        for n=1:NT
            area=area_f(x(n), t, area_index);
            LAx(n)=areax_f(x(n), t, area_index)/area;                       % define area term
            LAt(n)=areat_f(x(n), t, area_index)/area;
        end
        shock_index
        
        case {21, 28}                                                       % Laval nozzle start
        pexit_final=0.7735;                                                 % b. c. at exit plane corresponding to shock at x=1.5
        %pexit_final=0.72;
        %pexit_final=0.2353;                                                % corresponds to supersonic flow, M=1.6 at exit plane

        kmax=0;
        IS(1)=0;
        
        b=pistonlaw(t);                                                     % call piston routine
        c=exitboundary(t);                                                  % call exit boundary routine

        Xx=1/(c-b);
        
        gas_at_rest
        
        for n=1:NT
            area=area_f(x(n), t, area_index);
            LAx(n)=areax_f(x(n), t, area_index)/area;                       % define area term
            LAt(n)=areat_f(x(n), t, area_index)/area;
        end
        
        case 27

        Area_throat=2;                                                      % used in exact_Q1D_s
        x_throat=1;                                                         % used in exact_Q1D_s

       
        kmax=1;

        b=pistonlaw(t);                                                     % call piston routine
        c=exitboundary(t);                                                  % call exit boundary routine
        bt=pistonlawt(t);
        ct=exitboundaryt(t);
        Xx=1/(c-b);
        for n=1:NT
            X(n)=dx*(n-1);                                                  % computational coordinate                                             % contravariant velocity
        end
        Xt=-Xx*(bt+X*(ct-bt));                                              % derivative of X wrt t
        x=b+X/Xx;                                                           % physical coordinate
        
        exact_Q1D_s
        
        pexit_final=exp(P(NT));                                             
        U=Xt+u*Xx; 
        for n=1:NT
            area=area_f(x(n), t, area_index);
            LAx(n)=areax_f(x(n), t, area_index)/area;                       % define area term
            LAt(n)=areat_f(x(n), t, area_index)/area;
        end
        shock_index

end

R=2*a/gm+u;
Q=2*a/gm-u;
if kmax > 0
    RR=2*aR/gm+uR;
    RL=2*aL/gm+uL;
    QR=2*aR/gm-uR;
    QL=2*aL/gm-uL;
end

function gas_at_rest
% GAS_AT_REST.M - defines initial conditions
% A SHOCK FITTING PRIMER - CHAPTER 4, "Piston Driven Flows"
%
% Copyright 2005-2008 M.D.Salas.

global dx NT sg x X a u P S Z Xx Xt U t

b=pistonlaw(t);                                                             % call piston routine
bt=pistonlawt(t);                                                           % call exit boundary routine
ct=exitboundaryt(t);

for n=1:NT
    X(n)=dx*(n-1);                                                          % computational coordinate                                             % contravariant velocity
end
Xt=-Xx*(bt+X*(ct-bt));                                                      % derivative of X wrt t
x=b+X/Xx;                                                                   % physical coordinate
u(:)=0;                                                                     % velocity
P(:)=0;                                                                     % log of pressure
S(:)=0;                                                                     % entropy
Z(:)=1;                                                                     % temperature
a(:)=sg;                                                                    % speed of sound
U=Xt+u*Xx;                                                                  % contravariant velocity

function [p2,rho2,Z2,S2,a2,u2,wn]=rankine_hugoniot(SM1rel,u1,a1,p1,rho1,sign0)
% RANKINE_HUGONOIT.M - shock jump conditions
% A SHOCK FITTING PRIMER - CHAPTER 4, "Piston Driven Flows"
%
% Copyright 2005-2008 M.D.Salas.

global gamma gm gp

wn=u1+sign0*sqrt(SM1rel)*a1;
p2=p1*(2*gamma*SM1rel-gm)/gp;                                       
rho2=rho1*gp*SM1rel/(gm*SM1rel+2);
Z2=p2/rho2;
S2=log(p2)-gamma*log(rho2);
a2=sqrt(gamma*Z2);
u2=wn+rho1*(u1-wn)/rho2;

function exact_Q1D
% EXACT.M - exact solution to Q1D flow
% A SHOCK FITTING PRIMER - CHAPTER 4, "Piston Driven Flows".
%
% Copyright 2005-2008 M.D.Salas.

global gamma NT x area_index Area_throat x_throat P u Z S a t gp gm

G2=gp/gm/2;
G1=(gp/2)^G2;
once=0;
tol=10^-7;

for n=1:NT
    area_ratio(n)=Area_throat/area_f(x(n), t, area_index);
end

M=0;
Mach(1)=M;
for n=2:NT

     if (x(n) > x_throat) && (once == 0)
        M=1.01;
        once=1;
     end
    
    for m=1:20
        B=1+gm*M^2/2;
        A=B^(-G2);
        F=area_ratio(n)-G1*M*A;                                             % Newton iteration
        FM=G1*A*(gm*G2*M^2/B-1);
        Mn=M-F/FM;
        err=abs(Mn-M);
%{
        disp(['m= ' num2str(m) ', err= ' num2str(err) ', M= ' num2str(M)...
            ', x(n)= ' num2str(x(n)) ', xt= ' num2str(x_throat)...
            ', Arat= ' num2str(area_ratio(n))])
%}
        if err < tol
            break
        end
        M=Mn;
    end
    if err > tol
        disp(['In exact_Q1D.m, err= ' num2str(err) ' n= ' num2str(n)])
    end
    Mach(n)=Mn;
    M=Mn;

end

pstag1=1;
Zstag1=1;
for n=1:NT
    B=1+gm*Mach(n)^2/2;
    pn=pstag1*B^(-gamma/gm);
    P(n)=log(pn);
    Z(n)=Zstag1/B;
    a(n)=sqrt(gamma*Z(n));    
    u(n)=Mach(n)*a(n);
    S(n)=0;
end

function exact_Q1D_s
% EXACT.M - exact solution to Q1D flow with shock inserted
% A SHOCK FITTING PRIMER - CHAPTER 4, "Piston Driven Flows".
%
% Copyright 2005-2008 M.D.Salas.

global gamma NT x area_index Area_throat x_throat P u Z S a N 
global shock_type x_s uL uR ZL ZR PL PR SL SR aL aR w t gm gp

G2=gp/gm/2;
G1=(gp/2)^G2;
once=0;
tol=10^-7;

for n=1:NT
    area_ratio(n)=Area_throat/area_f(x(n), t, area_index);
end

M=0;
Mach(1)=M;
for n=2:NT
     area_rat=area_ratio(n);
     if (x(n) > x_throat) && (once == 0)
        M=1.01;
        once=1;
     end
    Mach(n)=find_mach(gm,M,G1,G2,area_rat,tol,n);
    M=Mach(n);
end
pstag1=1;
Zstag1=1;
rhostag1=1;
for n=1:NT
    B=1+gm*Mach(n)^2/2;
    pn=pstag1*B^(-gamma/gm);
    P(n)=log(pn);
    Z(n)=Zstag1/B;
    a(n)=sqrt(gamma*Z(n));
    u(n)=Mach(n)*a(n);
    S(n)=0;
end

xs=1.5;                                                                     % shock placed at x=1.5

b=pistonlaw(t);                            
c=exitboundary(t);
ns=1+fix(N*(xs-b)/(c-b));
x_s(1)=(x(ns)+x(ns+1))/2;                                                   % shock located in interval [ns,ns+1]
w(1)=0;
shock_type(1)='r';

area_ratio_s=Area_throat/area_f(x_s(1), t, area_index);                     % find solution to left of shock
M=Mach(ns);
ML=find_mach(gm,M,G1,G2,area_ratio_s,tol,ns);

B=1+gm*ML^2/2;
p1=pstag1*B^(-gamma/gm);
PL(1)=log(p1);
ZL(1)=Zstag1/B;
rho1=rhostag1*B^(-1/gm);
aL(1)=sqrt(gamma*ZL(1));
uL(1)=ML*aL(1);
SL(1)=0;

p2=p1*(2*gamma*ML^2-gm)/gp;                                                 % R-H
rho2=rho1*gp*ML^2/(gm*ML^2+2);
PR(1)=log(p2);
SR(1)=PR(1)-gamma*log(rho2);
ZR(1)=p2/rho2;
aR(1)=sqrt(gamma*ZR(1));
M2=sqrt((gm*ML^2 +2)/(2*gamma*ML^2-gm));
uR(1)=M2*aR(1);

Area_throat2=area_f(x_s(1), t, area_index)*G1*M2*(1+gm*M2^2/2)^(-G2);       % re-define throat

for n=ns+1:NT
    area_ratio(n)=Area_throat2/area_f(x(n), t, area_index);
end

M=M2;
for n=ns+1:NT
    area_rat=area_ratio(n);    
    Mach(n)=find_mach(gm,M,G1,G2,area_rat,tol,n);
    M=Mach(n);
end

pstag2=p1*((gp*ML^2/2)^(gamma/gm))*(gp/(2*gamma*ML^2-gm))^(1/gm);
rhostag2=pstag2*rhostag1/pstag1;
Zstag2=pstag2/rhostag2;

for n=ns+1:NT
    B=1+gm*Mach(n)^2/2;
    pn=pstag2*B^(-gamma/gm);
    P(n)=log(pn);
    Z(n)=Zstag2/B;
    a(n)=sqrt(gamma*Z(n));
    u(n)=Mach(n)*a(n);
    S(n)=SR(1);
end

function Mn=find_mach(gm,M,G1,G2,area_rat,tol,n)
global x x_throat

    for m=1:20
        B=1+gm*M^2/2;
        A=B^(-G2);
        F=area_rat-G1*M*A;                                                  % Newton iteration
        FM=G1*A*(gm*G2*M^2/B-1);
        Mn=M-F/FM;
        err=abs(Mn-M);
%{
        disp(['m= ' num2str(m) ', err= ' num2str(err) ', M= ' num2str(M)...
            ', x(n)= ' num2str(x(n)) ', xt= ' num2str(x_throat)...
            ', Arat= ' num2str(area_rat)])
%}
        if err < tol
            break
        end
        M=Mn;
    end
    if err > tol
        disp(['In exact_Q1D_s.m, err= ' num2str(err) ' n= ' num2str(n)])
    end