% FREQ_STUDY.M
% function to transfer time domain data to frequency domain
% reads data from a previous run of Q1D
% data is interpolated to equally spaced dt
% a portion of the data, from start_time to end_time is transformed to
% frequency domain using MATLAB fft function
% 
% A SHOCK FITTING PRIMER - CHAPTER 4, "Piston Driven Flows".
%
% Copyright 2005-2009 M.D.Salas.

clear all

global nfig

pr=dlmread( 'prdata.txt');                                                  % read high pressure side of shock data
pl=dlmread( 'pldata.txt');                                                  % read low pressure side of shock data
t=dlmread( 'tdata.txt');                                                    % read time at which data was recorded, not evenly spaced
x_s=dlmread( 'x_sdata.txt');                                                % read shock location
x_sonic=dlmread( 'x_sonicdata.txt');                                        % read sonic line location
t_sonic=dlmread( 't_sonicdata.txt');                                        % read time for sonic line location data

Npoints=length(t);                                                          % data vector length
NPsonic=length(t_sonic);

disp(['number of data samples=' num2str(Npoints)]);
disp(' ')
disp('turning points for high pressure')
m=0;
for n=2:Npoints-1                                                           % find period in time domain
    if (pr(n)-pr(n-1))*(pr(n+1)-pr(n)) < 0 
        m=m+1;                                                              
        disp(['m= ' num2str(m) ', time= ' num2str(t(n)) ', pr= '...
            num2str(pr(n)) ', n= ' num2str(n)]);  
        prm(m)=pr(n);                                                       % save data for plotting
        tprm(m)=t(n);
    end
end
disp(' ')
disp('turning points for low pressure')
m=0;
for n=2:Npoints-1 
    if (pl(n)-pl(n-1))*(pl(n+1)-pl(n)) < 0
        m=m+1;
        disp(['m= ' num2str(m) ', time= ' num2str(t(n)) ', pl= '...
            num2str(pl(n)) ', n= ' num2str(n)]);  
        plm(m)=pl(n);                                                       
        tplm(m)=t(n);
    end
end

disp(' ')
disp('turning points for shock location')
m=0;
for n=2:Npoints-1 
    if (x_s(n)-x_s(n-1))*(x_s(n+1)-x_s(n)) < 0  
        m=m+1;
        disp(['m= ' num2str(m) ', time= ' num2str(t(n)) ', x_s(n)= '...
            num2str(x_s(n)) ', n= ' num2str(n)]);
        x_sm(m)=x_s(n);                                                       
        tx_sm(m)=t(n);
    end
end

nfig=1;
figure(nfig)                                                                % high pressure data
plot(t,pr,'k-', tprm,prm,'ro', 'LineWidth', 2)
ylim([.53 .57 ])
grid on
xlabel('time')
ylabel('high pressure')
title('shock wave high pressure data in time domain')

nfig=nfig+1;
figure(nfig)                                                                % low pressure data
plot(t,pl,'k-', tplm,plm,'ro', 'LineWidth', 2)
grid on
ylim([.49 .51 ])
xlabel('time')
ylabel('low pressure')
title('shock wave low pressure data in time domain')

nfig=nfig+1;
figure(nfig)                                                                % shock & sonic line path data
plot(t,x_s,'k-',tx_sm,x_sm,'ro', 'LineWidth', 2)
hold on

threshold=0.01;
m=0;
 for n=2:NPsonic                                                            % the sonic line has several breaks in it.
     if abs(x_sonic(n)-x_sonic(n-1)) < threshold                            % we look for the breakes and plot each section
         m=m+1;
         tplot(m)=t_sonic(n);
         xplot(m)=x_sonic(n);
     else
         m=0;
         plot(tplot,xplot,'r-', 'LineWidth', 2)
         clear tplot xplot
     end
     if m > 0
         plot(tplot,xplot,'r-', 'LineWidth', 2)
     end
 end
grid on
xlabel('time')
ylabel('shock and sonic line location')
title('shock wave and sonic line location in time domain')
hold off
disp(' ')

ttotal=t(Npoints)-t(1);
disp(['starting time for data sample=' num2str(t(1))]);
disp(['time interval for data sample=' num2str(ttotal)]);
dtaverage=ttotal/Npoints;
disp(['data sample average dt=' num2str(dtaverage)]);


L=2000;                                                                     % number of samples used with fft
start_time=40;                                                              % start time for fft analysis
end_time=t(Npoints);                                                        % end time for fft analysis
time_interval=end_time-start_time;
dt=time_interval/(L-1);                                                     % sample time
Fs=2000;                                                                    % Sampling frequency
disp(['equally spaced delta t used to sample signal=' num2str(dt)]);
disp(' ')

for n=1:L                                                                   % define equally space time
t1(n)=start_time+(n-1)*dt;
end

for n=1:L                                                                   % interpolate original data at equally spaced times
pr1(n)=interp1(t,pr,t1(n),'cubic');
pl1(n)=interp1(t,pl,t1(n),'cubic');
x_s1(n)=interp1(t,x_s,t1(n),'cubic');
end

nfig=nfig+1;
figure(nfig)                                                                % high pressure data
plot(t1,pr1,'k-')
ylim([.535 .57 ])
grid on
xlabel('time')
ylabel('high pressure')
title('shock wave high pressure data in time domain, data sent to fft')

nfig=nfig+1;
figure(nfig)                                                                % low pressure data
plot(t1,pl1,'k-')
grid on
ylim([.49 .508 ])
xlabel('time')
ylabel('low pressure')
title('shock wave low pressure data in time domain, data sent to fft')

nfig=nfig+1;
figure(nfig)                                                                % shock path data
plot(t1,x_s1,'k-')
grid on
ylim([1.014 1.032 ])
xlabel('time')
ylabel('shock location')
title('shock wave location in time domain, data sent to fft')
disp(' ')

freq(L, Fs, time_interval, pr1)

f2=1/(tprm(13)-tprm(11));
disp([ 'shock high pressure dominant frequency from time domain, f2= ' num2str(f2) ])
disp(' ')


freq(L, Fs, time_interval, pl1)

f2=1/(tplm(35)-tplm(29));
disp([ 'shock low pressure dominant frequency from time domain, f2= ' num2str(f2) ])
disp(' ')

freq(L, Fs, time_interval, x_s1)

M=length(tx_sm);
f2=1/(tx_sm(M)-tx_sm(M-2));
disp([ 'shock location dominant frequency from time domain, f2= ' num2str(f2) ])
