%     program supersonic wing
%     flow must be supersonic between body and shock
%     valid only for elliptical cross sections
%
% Copyright (c) 2005- 2009 by M.D.Salas
%
clear all

global na ma ka ja nc ncm mc mcm mcp  m mach gamma ga gb gc gd ge gf
global cone z1 pio2 vin hst z dx dy dz hh zn vl2 vl3 attack
global hhn h2zn h2z k j nfig gindex istore Ivisc
global cz bz c b x y ch sinthe costhe r p s w u v t
global cossin guess_shk_ang ttotal yz h2zz h2zzn zlast

%
%     see subroutine sw_geom for geometry definition
%
%     nrun=run number
%     na=no. of intervals between shock and body
%     ma=no. of intervals between windward and leeward planes
%     ka=no. of steps in z (marching) direction
%     ja=output frequency
%     loadx=1 to load data from previous run (use only for conical input)
%     loadx=0 no load
%     iadd=0 no points added, iadd=1 points doubled
%     gindex=1 ogive cylinder, gindex=2 elliptical wing, gindex=3 circular
%     cone degree, uses cone as the cone angle
%     Ivisc=0 no artificial dissipation.
%     Ivisc=1 adds artificial dissipation. Use only for very weak shocks and
%             only if absolutely necessary to avoid wiggles due to singularity
%             at shock.
%     mach=free stream Mach no. (large enough for attached shock)
%     gamma=isentropic exponent
%     cone=windward plane cone half angle in degrees (used to define initial
%           conditions)
%     dist=distance (z direction) to end of computation
%     attack=angle of attack in degrees, start with attack =0 and increase
%           attack by incrementally raising it while marching in z direction
%     guess_shk_ang=guess for shock wave angle in degrees, windward plane
%           (use cone shock tables: chart 5 of NACA Report 1135, 1953)
%           if guess_shk_ang=-1 use build in guess valid for mach >= 2
%
nrun=1;
na=40;
ma=40;
ka=1;
ja=100;
loadx=0;
iadd=0;
gindex=1;
Ivisc=0;
mach=4;
gamma=1.4;
cone=10;
dist=20;
attack=0;
guess_shk_ang=-1; % example: 30.094 deg for Mach=2, cone=5,
%                            19.715 deg for Mach=3, cone=5
%
% NO CHANGES BELOW THIS LINE
%
disp(' ')
disp('PROGRAM SUPERSONIC WING')
disp(['Run Number ',num2str(nrun)])
disp(['Mach= ' num2str(mach) ', gamma= ' num2str(gamma) ', cone= ' num2str(cone)...
    ', dist= ' num2str(dist) ', attack= ' num2str(attack) ])
disp(['na= ' num2str(na) ', ma= ' num2str(ma) ', ka= ' num2str(ka)])
%
if guess_shk_ang == -1 && mach < 2
    disp('The build in guess for shock angle requires mach >= 2.')
    disp('Use NACA Report 1135 to guess shock angle.')
    break
end;
z1=dist;
nfig=0;
stab=.7;
mc=ma+2;
nc=na+1;
ncm=na;
mcm=mc-1;
mcp=mc+1;
gb=1./(gamma-1.);
ga=gamma*gb;
gd=.5/gb;
ge=gd+1.;
gc=ge/gd;
gf=sqrt(gamma);
ttotal=1.+gd* mach^2;
vin=gf*mach;
hst=ga+.5*vin^2;
pio2=pi/2;
attack=attack*pi/180.;
cone=cone*pi/180.;
vl3=cos(attack);
vl2=sin(attack);
dx=1/na;
dy=pi/ma;
for m=1:mcp;
    y(m)=dy*(m-2);
    costhe(m)=sin(y(m));
    sinthe(m)=-cos(y(m));
    cossin(m)=costhe(m)^2-sinthe(m)^2;
end
%
sw_geom(0); % call geometry to get geometric parameters
zn=0; % initial plane
sw_geom(1); % call geometry to get shape
z=0;
hh=hhn;
h2z=h2zn;
h2zz=h2zzn;
%
if loadx == 0;
    %
    %       initial solution for a sharp circular cone
    %       solves Taylor-Maccoll equations
    %
    sw_initial;
else
    %
    %       load solution from file: sw_save.mat
    %
    load sw_save.mat u v w p s t cz c zlast
    disp('Solution loaded from file.')
    sw_scale; % scales solution by a factor of ztar/(zlast+zstar)
    %
    if iadd == 1; % double mesh size
        sw_addpoints;
        disp('Mesh points increased.')
        disp(['na=' num2str(na) ', ma=' num2str(ma)])
    end
end
%
%       integration begins here
%
j=0;
k=0;
sw_output;
%
%datt=.5*pi/180/(ka-1);
%attack0=attack;
for k=1:ka;
    %attack=attack0+(k-1)*datt;
    %vl3=cos(attack);
    %vl2=sin(attack);
    
    j=j+1;
    %
    %       marching step computed according to cfl rule
    %
    dz=1.;
    for  m=2:mc;
        for  n=1:nc;
            t(n,m)=exp(s(n,m)/gamma+p(n,m)/ga);
            r(n,m)=b(m)+(c(m)-b(m))*x(n);
            h2or2=(hh/r(n,m))^2;
            xr=1./(c(m)-b(m));
            xz=xr*(bz(m)*(x(n)-1.)-cz(m)*x(n));
            xh=-xr*ch(m)*x(n);
            xxr=(1.-h2or2)*costhe(m);
            yyr=(1.+h2or2)*sinthe(m);
            xxh=-r(n,m)*(1.+h2or2)*sinthe(m);
            yyh=r(n,m)*xxr;
            xxz=h2z*costhe(m)/r(n,m);
            yyz=-h2z*sinthe(m)/r(n,m);
            del=1.+(h2or2-2.*cossin(m))*h2or2;
            dum=h2z/r(n,m);
            rx=xxr/del;
            ry=yyr/del;
            hx=-ry/r(n,m);
            hy=rx/r(n,m);
            rz=-(ry*yyz+rx*xxz);
            hz=-(hy*yyz+hx*xxz);
            xxx=xr*rx+xh*hx;
            xyy=xr*ry+xh*hy;
            xzz=xz+xr*rz+xh*hz;
            yxx=hx;
            yyy=hy;
            yzz=yz(m)+hz;
            ad=w(n,m)^2-gamma*t(n,m);
            ax=xzz+(u(n,m)*xxx+v(n,m)*xyy)*w(n,m)/ad;
            ay=yzz+(u(n,m)*yxx+v(n,m)*yyy)*w(n,m)/ad;
            az=1.+u(n,m)^2/ad;
            aw=u(n,m)*v(n,m)/ad;
            at=1.+v(n,m)^2/ad;
            bx=t(n,m)*(xxx*az+xyy*aw)*xxx*gamma/ad;
            by=t(n,m)*(yxx*az+yyy*aw)*yxx*gamma/ad;
            cx=t(n,m)*(xyy*at+xxx*aw)*xyy*gamma/ad;
            cy=t(n,m)*(yyy*at+yxx*aw)*yyy*gamma/ad;
            xlxu=ax+sqrt(bx+cx);
            xlxl=ax-sqrt(bx+cx);
            xlyu=ay+sqrt(by+cy);
            xlyl=ay-sqrt(by+cy);
            dzxu=abs(dx/xlxu);
            dzxl=abs(dx/xlxl);
            dzyu=abs(dy/xlyu);
            dzyl=abs(dy/xlyl);
            dzvalue=[dzxu,dzyu,dzxl,dzyl,dz];
            dz=min(dzvalue);
        end % for n loop
    end % for m loop
    dz=stab*dz;
    zn=z+dz;
    if zn >= z1;
        zn=z1;
        dz=zn-z;
    end
    sw_geom(1);
    %{
    % sw_points_w uses Bernoulli's equation to solve for w
    % sw_points_w;
    %}
    sw_points;
    z=zn;
    %
    %       store data for plotting
    %
    istore=0;
    sw_store;
    %
    if j == ja;
        j=0;
        sw_output
    end
    
    if z >= dist;
        sw_output
        break
    end
end % for k loop
%
%       save data for future run
%
zlast=z;
save sw_save_s.mat u v w p s t cz c zlast
disp('Solution saved to file.')
%
%       plot final data
%
istore=1;
sw_store
sw_display

%
disp(['Computation ends at z= ',num2str(z) ' k= ' ,num2str(k)])
disp(['The angle-of-attack is ',num2str(attack*180/pi)])